@@ -23,8 +23,8 @@
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
+#include <arpa/inet.h>
 
-#include <netatalk/endian.h>
 #include <atalk/unicode.h>
 #include <atalk/logger.h>
 
@@ -40,6 +40,7 @@ struct charset_functions charset_iso8859_adapted =
 	iso8859_adapted_pull,
 	iso8859_adapted_push,
 	CHARSET_MULTIBYTE | CHARSET_PRECOMPOSED,
+	NULL,
 	NULL, NULL
 };
 
@@ -48,7 +49,7 @@ struct charset_functions charset_iso8859_adapted =
  * from unicode to iso8859_adapted code page
 */
 
-static size_t iso8859_adapted_push( void *cd, char **inbuf, size_t *inbytesleft,
+static size_t iso8859_adapted_push( void *cd _U_, char **inbuf, size_t *inbytesleft,
                          char **outbuf, size_t *outbytesleft)
 {
     int len = 0;
@@ -102,7 +103,7 @@ static size_t iso8859_adapted_push( void *cd, char **inbuf, size_t *inbytesleft,
 }
 
 /* ------------------------ */
-static size_t iso8859_adapted_pull ( void *cd, char **inbuf, size_t *inbytesleft,
+static size_t iso8859_adapted_pull ( void *cd _U_, char **inbuf, size_t *inbytesleft,
                          char **outbuf, size_t *outbytesleft)
 {
     unsigned char  *inptr;
