@@ -1,6 +1,7 @@
 /* 
  * Netatalk 2002 (c)
  * Copyright (C) 1990, 1993 Regents of The University of Michigan
+ * Copyright (C) 2010 Frank Lahm
  * All Rights Reserved. See COPYRIGHT
  */
 
@@ -18,6 +19,10 @@
  *
  * Initial version written by Rafal Lewczuk <rlewczuk@pronet.pl>
  *
+ * Starting with Netatalk 2.2 searching by name criteria utilizes the
+ * CNID database in conjunction with an enhanced cnid_dbd. This requires
+ * the use of cnidscheme:dbd for the searched volume, the new functionality
+ * is not built into cnidscheme:cdb.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -26,44 +31,39 @@
 
 #include <stdio.h>
 #include <stdlib.h>
-#include <dirent.h>
 #include <errno.h>
 #include <ctype.h>
 #include <string.h>
 #include <time.h>
-
-#if STDC_HEADERS
 #include <string.h>
-#else
-#ifndef HAVE_MEMCPY
-#define memcpy(d,s,n) bcopy ((s), (d), (n))
-#define memmove(d,s,n) bcopy ((s), (d), (n))
-#endif /* ! HAVE_MEMCPY */
-#endif
-
 #include <sys/file.h>
 #include <netinet/in.h>
 
 #include <atalk/afp.h>
 #include <atalk/adouble.h>
 #include <atalk/logger.h>
-#ifdef CNID_DB
 #include <atalk/cnid.h>
-#endif /* CNID_DB */
+#include <atalk/cnid_dbd_private.h>
+#include <atalk/util.h>
+#include <atalk/bstradd.h>
+#include <atalk/unicode.h>
+#include <atalk/globals.h>
+#include <atalk/netatalk_conf.h>
+
 #include "desktop.h"
 #include "directory.h"
+#include "dircache.h"
 #include "file.h"
 #include "volume.h"
-#include "globals.h"
 #include "filedir.h"
 #include "fork.h"
 
 
 struct finderinfo {
-	u_int32_t f_type;
-	u_int32_t creator;
-	u_int16_t attrs;    /* File attributes (high 8 bits)*/
-	u_int16_t label;    /* Label (low 8 bits )*/
+	uint32_t f_type;
+	uint32_t creator;
+	uint16_t attrs;    /* File attributes (high 8 bits)*/
+	uint16_t label;    /* Label (low 8 bits )*/
 	char reserved[22]; /* Unknown (at least for now...) */
 };
 
@@ -87,17 +87,17 @@ typedef char packed_finder[ADEDLEN_FINDERI];
 
 /* This is our search-criteria structure. */
 struct scrit {
-	u_int32_t rbitmap;          /* Request bitmap - which values should we check ? */
-	u_int16_t fbitmap, dbitmap; /* file & directory bitmap - which values should we return ? */
-	u_int16_t attr;             /* File attributes */
+	uint32_t rbitmap;          /* Request bitmap - which values should we check ? */
+	uint16_t fbitmap, dbitmap; /* file & directory bitmap - which values should we return ? */
+	uint16_t attr;             /* File attributes */
 	time_t cdate;               /* Creation date */
 	time_t mdate;               /* Last modification date */
 	time_t bdate;               /* Last backup date */
-	u_int32_t pdid;             /* Parent DID */
-    u_int16_t offcnt;           /* Offspring count */
+	uint32_t pdid;             /* Parent DID */
+    uint16_t offcnt;           /* Offspring count */
 	struct finderinfo finfo;    /* Finder info */
 	char lname[64];             /* Long name */ 
-	char utf8name[512];         /* UTF8 name */
+	char utf8name[514];         /* UTF8 or UCS2 name */ /* for convert_charset dest_len parameter +2 */
 };
 
 /*
@@ -109,58 +109,75 @@ struct scrit {
  *
  */
 struct dsitem {
-	struct dir *dir; /* Structure describing this directory */
+//	struct dir *dir; /* Structure describing this directory */
+//  cnid_t did;      /* CNID of this directory           */
 	int pidx;        /* Parent's dsitem structure index. */
 	int checked;     /* Have we checked this directory ? */
+	int path_len;
 	char *path;      /* absolute UNIX path to this directory */
 };
  
 
 #define DS_BSIZE 128
-static u_int32_t cur_pos = 0;    /* Saved position index (ID) - used to remember "position" across FPCatSearch calls */
-static DIR *dirpos = NULL; /* UNIX structure describing currently opened directory. */
 static int save_cidx = -1; /* Saved index of currently scanned directory. */
-
 static struct dsitem *dstack = NULL; /* Directory stack data... */
 static int dssize = 0;  	     /* Directory stack (allocated) size... */
 static int dsidx = 0;   	     /* First free item index... */
-
 static struct scrit c1, c2;          /* search criteria */
 
+/* Clears directory stack. */
+static void clearstack(void) 
+{
+	save_cidx = -1;
+	while (dsidx > 0) {
+		dsidx--;
+		free(dstack[dsidx].path);
+	}
+}
+
 /* Puts new item onto directory stack. */
 static int addstack(char *uname, struct dir *dir, int pidx)
 {
 	struct dsitem *ds;
-	int           l;
+	size_t         l, u;
+    struct dsitem *tmpds = NULL;
 
 	/* check if we have some space on stack... */
 	if (dsidx >= dssize) {
 		dssize += DS_BSIZE;
-		dstack = realloc(dstack, dssize * sizeof(struct dsitem));	
-		if (dstack == NULL)
+		tmpds = realloc(dstack, dssize * sizeof(struct dsitem));	
+		if (tmpds == NULL) {
+            clearstack();
+            free(dstack);
 			return -1;
+        }
+        dstack = tmpds;
 	}
 
 	/* Put new element. Allocate and copy lname and path. */
 	ds = dstack + dsidx++;
-	ds->dir = dir;
+//	ds->did = dir->d_did;
 	ds->pidx = pidx;
-	if (pidx >= 0) {
-	    l = strlen(dstack[pidx].path);
-	    if (!(ds->path = malloc(l + strlen(uname) + 2) ))
-			return -1;
-		strcpy(ds->path, dstack[pidx].path);
-		strcat(ds->path, "/");
-		strcat(ds->path, uname);
-	}
-
 	ds->checked = 0;
-
+	if (pidx >= 0) {
+	    l = dstack[pidx].path_len;
+	    u = strlen(uname) +1;
+	    if (!(ds->path = malloc(l + u + 1) ))
+			return -1;
+		memcpy(ds->path, dstack[pidx].path, l);
+		ds->path[l] = '/';
+		memcpy(&ds->path[l+1], uname, u);
+		ds->path_len = l +u;
+	}
+	else {
+	    ds->path = strdup(uname);
+		ds->path_len = strlen(uname);
+	}
 	return 0;
 }
 
 /* Removes checked items from top of directory stack. Returns index of the first unchecked elements or -1. */
-static int reducestack()
+static int reducestack(void)
 {
 	int r;
 	if (save_cidx != -1) {
@@ -179,16 +196,6 @@ static int reducestack()
 	return -1;
 } 
 
-/* Clears directory stack. */
-static void clearstack() 
-{
-	save_cidx = -1;
-	while (dsidx > 0) {
-		dsidx--;
-		free(dstack[dsidx].path);
-	}
-} 
-
 /* Looks up for an opened adouble structure, opens resource fork of selected file. 
  * FIXME What about noadouble?
 */
@@ -207,11 +214,11 @@ static struct adouble *adl_lkup(struct vol *vol, struct path *path, struct adoub
 	if (!isdir && (of = of_findname(path))) {
 		adp = of->of_ad;
 	} else {
-		ad_init(&ad, vol->v_adouble, vol->v_ad_options);
+		ad_init(&ad, vol);
 		adp = &ad;
 	} 
 
-    if ( ad_metadata( path->u_name, ((isdir)?ADFLAGS_DIR:0), adp) < 0 ) {
+    if ( ad_metadata( path->u_name, ((isdir) ? ADFLAGS_DIR : 0), adp) < 0 ) {
         adp = NULL; /* FIXME without resource fork adl_lkup will be call again */
     }
     
@@ -233,12 +240,13 @@ static struct finderinfo *unpack_buffer(struct finderinfo *finfo, char *buffer)
 
 /* -------------------- */
 static struct finderinfo *
-unpack_finderinfo(char *upath, struct adouble *adp, struct finderinfo *finfo)
+unpack_finderinfo(struct vol *vol, struct path *path, struct adouble **adp, struct finderinfo *finfo, int islnk)
 {
 	packed_finder  buf;
 	void           *ptr;
 	
-	ptr = get_finderinfo(upath, adp, &buf);
+    *adp = adl_lkup(vol, path, *adp);
+	ptr = get_finderinfo(vol, path->u_name, *adp, &buf,islnk);
 	return unpack_buffer(finfo, ptr);
 }
 
@@ -253,13 +261,15 @@ unpack_finderinfo(char *upath, struct adouble *adp, struct finderinfo *finfo)
  */
 static int crit_check(struct vol *vol, struct path *path) {
 	int result = 0;
-	u_int16_t attr, flags = CONV_PRECOMPOSE;
+	uint16_t attr, flags = CONV_PRECOMPOSE;
 	struct finderinfo *finfo = NULL, finderinfo;
 	struct adouble *adp = NULL;
 	time_t c_date, b_date;
-	u_int32_t ac_date, ab_date;
-	static char convbuf[512];
+	uint32_t ac_date, ab_date;
+	static char convbuf[514]; /* for convert_charset dest_len parameter +2 */
 	size_t len;
+    int islnk;
+    islnk=S_ISLNK(path->st.st_mode);
 
 	if (S_ISDIR(path->st.st_mode)) {
 		if (!c1.dbitmap)
@@ -274,7 +284,7 @@ static int crit_check(struct vol *vol, struct path *path) {
 		 * An other option would be to call get_id in utompath but 
 		 * we need to pass parent dir
 		*/
-        if (!(path->m_name = utompath(vol, path->u_name, 0 , utf8_encoding()) )) {
+        if (!(path->m_name = utompath(vol, path->u_name, 0 , utf8_encoding(vol->v_obj)) )) {
         	/*retry with the right id */
        
         	cnid_t id;
@@ -285,7 +295,9 @@ static int crit_check(struct vol *vol, struct path *path) {
         		/* FIXME */
         		return 0;
         	}
-        	if (!(path->m_name = utompath(vol, path->u_name, id , utf8_encoding()))) {
+        	/* save the id for getfilparm */
+        	path->id = id;
+        	if (!(path->m_name = utompath(vol, path->u_name, id , utf8_encoding(vol->v_obj)))) {
         		return 0;
         	}
         }
@@ -299,9 +311,9 @@ static int crit_check(struct vol *vol, struct path *path) {
 
 	/* Check for filename */
 	if ((c1.rbitmap & (1<<DIRPBIT_LNAME))) { 
-		if ( (size_t)(-1) == (len = convert_string(vol->v_maccharset, CH_UCS2, path->m_name, strlen(path->m_name), convbuf, 512)) )
+		if ( (size_t)(-1) == (len = convert_string(vol->v_maccharset, CH_UCS2, path->m_name, -1, convbuf, 512)) )
 			goto crit_check_ret;
-		convbuf[len] = 0; 
+
 		if ((c1.rbitmap & (1<<CATPBIT_PARTIAL))) {
 			if (strcasestr_w( (ucs2_t*) convbuf, (ucs2_t*) c1.lname) == NULL)
 				goto crit_check_ret;
@@ -314,7 +326,7 @@ static int crit_check(struct vol *vol, struct path *path) {
 		if ( (size_t)(-1) == (len = convert_charset( CH_UTF8_MAC, CH_UCS2, CH_UTF8, path->m_name, strlen(path->m_name), convbuf, 512, &flags))) {
 			goto crit_check_ret;
 		}
-		convbuf[len] = 0; 
+
 		if (c1.rbitmap & (1<<CATPBIT_PARTIAL)) {
 			if (strcasestr_w((ucs2_t *) convbuf, (ucs2_t*)c1.utf8name) == NULL)
 				goto crit_check_ret;
@@ -372,8 +384,7 @@ static int crit_check(struct vol *vol, struct path *path) {
 
         /* Check file type ID */
 	if ((c1.rbitmap & (1<<DIRPBIT_FINFO)) && c2.finfo.f_type != 0) {
-		adp = adl_lkup(vol, path, adp);
-	    finfo = unpack_finderinfo(path->u_name, adp, &finderinfo);
+	    finfo = unpack_finderinfo(vol, path, &adp, &finderinfo,islnk);
 		if (finfo->f_type != c1.finfo.f_type)
 			goto crit_check_ret;
 	}
@@ -381,8 +392,7 @@ static int crit_check(struct vol *vol, struct path *path) {
 	/* Check creator ID */
 	if ((c1.rbitmap & (1<<DIRPBIT_FINFO)) && c2.finfo.creator != 0) {
 		if (!finfo) {
-	    	adp = adl_lkup(vol, path, adp);
-			finfo = unpack_finderinfo(path->u_name, adp, &finderinfo);
+			finfo = unpack_finderinfo(vol, path, &adp, &finderinfo,islnk);
 		}
 		if (finfo->creator != c1.finfo.creator)
 			goto crit_check_ret;
@@ -391,8 +401,7 @@ static int crit_check(struct vol *vol, struct path *path) {
 	/* Check finder info attributes */
 	if ((c1.rbitmap & (1<<DIRPBIT_FINFO)) && c2.finfo.attrs != 0) {
 		if (!finfo) {
-	    	adp = adl_lkup(vol, path, adp);
-			finfo = unpack_finderinfo(path->u_name, adp, &finderinfo);
+			finfo = unpack_finderinfo(vol, path, &adp, &finderinfo,islnk);
 		}
 
 		if ((finfo->attrs & c2.finfo.attrs) != c1.finfo.attrs)
@@ -402,8 +411,7 @@ static int crit_check(struct vol *vol, struct path *path) {
 	/* Check label */
 	if ((c1.rbitmap & (1<<DIRPBIT_FINFO)) && c2.finfo.label != 0) {
 		if (!finfo) {
-	    	adp = adl_lkup(vol, path, adp);
-			finfo = unpack_finderinfo(path->u_name, adp, &finderinfo);
+			finfo = unpack_finderinfo(vol, path, &adp, &finderinfo,islnk);
 		}
 		if ((finfo->label & c2.finfo.label) != c1.finfo.label)
 			goto crit_check_ret;
@@ -419,12 +427,13 @@ crit_check_ret:
 }  
 
 /* ------------------------------ */
-static int rslt_add ( struct vol *vol, struct path *path, char **buf, int ext)
+static int rslt_add (const AFPObj *obj, struct vol *vol, struct path *path, char **buf, int ext)
 {
 
 	char 		*p = *buf;
-	int 		ret, tbuf =0;
-	u_int16_t	resultsize;
+	int 		ret;
+	size_t		tbuf =0;
+	uint16_t	resultsize;
 	int 		isdir = S_ISDIR(path->st.st_mode); 
 
 	/* Skip resultsize */
@@ -441,11 +450,11 @@ static int rslt_add ( struct vol *vol, struct path *path, char **buf, int ext)
 	}
 	
 	if ( isdir ) {
-        ret = getdirparams(vol, c1.dbitmap, path, path->d_dir, p , &tbuf ); 
+        ret = getdirparams(obj, vol, c1.dbitmap, path, path->d_dir, p , &tbuf ); 
 	}
 	else {
-	    /* FIXME slow if we need the file ID, we already know it */
-		ret = getfilparams ( vol, c1.fbitmap, path, path->d_dir, p, &tbuf);
+	    /* FIXME slow if we need the file ID, we already know it, done ? */
+		ret = getfilparams (obj, vol, c1.fbitmap, path, path->d_dir, p, &tbuf, 0);
 	}
 
 	if ( ret != AFP_OK )
@@ -473,31 +482,45 @@ static int rslt_add ( struct vol *vol, struct path *path, char **buf, int ext)
 #define VETO_STR \
         "./../.AppleDouble/.AppleDB/Network Trash Folder/TheVolumeSettingsFolder/TheFindByContentFolder/.AppleDesktop/.Parent/"
 
-/* This function performs search. It is called directly from afp_catsearch 
- * vol - volume we are searching on ...
- * dir - directory we are starting from ...
- * c1, c2 - search criteria
- * rmatches - maximum number of matches we can return
- * pos - position we've stopped recently
- * rbuf - output buffer
- * rbuflen - output buffer length
+/*!
+ * This function performs a filesystem search
+ *
+ * Uses globals c1, c2, the search criteria
+ *
+ * @param vol       (r)  volume we are searching on ...
+ * @param dir       (rw) directory we are starting from ...
+ * @param rmatches  (r)  maximum number of matches we can return
+ * @param pos       (r)  position we've stopped recently
+ * @param rbuf      (w)  output buffer
+ * @param nrecs     (w)  number of matches
+ * @param rsize     (w)  length of data written to output buffer
+ * @param ext       (r)  extended search flag
  */
-#define NUM_ROUNDS 100
-static int catsearch(struct vol *vol, struct dir *dir,  
-		     int rmatches, u_int32_t *pos, char *rbuf, u_int32_t *nrecs, int *rsize, int ext)
+#define NUM_ROUNDS 200
+static int catsearch(const AFPObj *obj,
+                     struct vol *vol,
+                     struct dir *dir,  
+                     int rmatches,
+                     uint32_t *pos,
+                     char *rbuf,
+                     uint32_t *nrecs,
+                     int *rsize,
+                     int ext)
 {
+    static uint32_t cur_pos;    /* Saved position index (ID) - used to remember "position" across FPCatSearch calls */
+    static DIR *dirpos; 		 /* UNIX structure describing currently opened directory. */
+    struct dir *currentdir;      /* struct dir of current directory */
 	int cidx, r;
 	struct dirent *entry;
 	int result = AFP_OK;
 	int ccr;
     struct path path;
-	char *orig_dir = NULL;
-	int orig_dir_len = 128;
 	char *vpath = vol->v_path;
 	char *rrbuf = rbuf;
     time_t start_time;
     int num_rounds = NUM_ROUNDS;
-    int cached;
+    int cwd = -1;
+    int error;
         
 	if (*pos != 0 && *pos != cur_pos) {
 		result = AFPERR_CATCHNG;
@@ -506,8 +529,6 @@ static int catsearch(struct vol *vol, struct dir *dir,
 
 	/* FIXME: Category "offspring count ! */
 
-	/* So we are beginning... */
-    start_time = time(NULL);
 
 	/* We need to initialize all mandatory structures/variables and change working directory appropriate... */
 	if (*pos == 0) {
@@ -517,32 +538,34 @@ static int catsearch(struct vol *vol, struct dir *dir,
 			dirpos = NULL;
 		} 
 		
-		if (addstack("", dir, -1) == -1) {
+		if (addstack(vpath, dir, -1) == -1) {
 			result = AFPERR_MISC;
 			goto catsearch_end;
 		}
-		dstack[0].path = strdup(vpath);
 		/* FIXME: Sometimes DID is given by client ! (correct this one above !) */
 	}
 
 	/* Save current path */
-	orig_dir = (char*)malloc(orig_dir_len);
-	while (getcwd(orig_dir, orig_dir_len-1)==NULL) {
-		if (errno != ERANGE) {
-			result = AFPERR_MISC;
-			goto catsearch_end;
-		}
-		orig_dir_len += 128; 
-		orig_dir = realloc(orig_dir, orig_dir_len);
-	} /* while() */
+    if ((cwd = open(".", O_RDONLY)) < 0) {
+        result = AFPERR_MISC;
+        goto catsearch_end;
+    }
 	
+	/* So we are beginning... */
+    start_time = time(NULL);
+
 	while ((cidx = reducestack()) != -1) {
-		cached = 1;
-		if (dirpos == NULL) {
+        LOG(log_debug, logtype_afpd, "catsearch: dir: \"%s\"", dstack[cidx].path);
+
+		error = lchdir(dstack[cidx].path);
+
+		if (!error && dirpos == NULL)
+			dirpos = opendir(".");
+
+		if (dirpos == NULL)
 			dirpos = opendir(dstack[cidx].path);
-			cached = (dstack[cidx].dir->d_child != NULL);
-		}
-		if (dirpos == NULL) {
+
+		if (error || dirpos == NULL) {
 			switch (errno) {
 			case EACCES:
 				dstack[cidx].checked = 1;
@@ -559,17 +582,23 @@ static int catsearch(struct vol *vol, struct dir *dir,
 			} /* switch (errno) */
 			goto catsearch_end;
 		}
-		/* FIXME error in chdir, what do we do? */
-		chdir(dstack[cidx].path);
-		
 
-		while ((entry=readdir(dirpos)) != NULL) {
+        if ((currentdir = dirlookup_bypath(vol, dstack[cidx].path)) == NULL) {
+            result = AFPERR_MISC;
+            goto catsearch_end;
+        }
+        LOG(log_debug, logtype_afpd, "catsearch: current struct dir: \"%s\"", cfrombstr(currentdir->d_fullpath));
+		
+		while ((entry = readdir(dirpos)) != NULL) {
 			(*pos)++;
 
 			if (!check_dirent(vol, entry->d_name))
 			   continue;
 
-			path.m_name = NULL;
+            LOG(log_debug, logtype_afpd, "catsearch(\"%s\"): dirent: \"%s\"",
+                cfrombstr(currentdir->d_fullpath), entry->d_name);
+
+			memset(&path, 0, sizeof(path));
 			path.u_name = entry->d_name;
 			if (of_stat(&path) != 0) {
 				switch (errno) {
@@ -591,33 +620,36 @@ static int catsearch(struct vol *vol, struct dir *dir,
 				   ie if in the same loop the parent dir wasn't in the cache
 				   ALL dirsearch_byname will fail.
 				*/
-				if (cached)
-            		path.d_dir = dirsearch_byname(dstack[cidx].dir, path.u_name);
-            	else
-            		path.d_dir = NULL;
-            	if (!path.d_dir) {
+                int unlen = strlen(path.u_name);
+                path.d_dir = dircache_search_by_name(vol,
+                                                     currentdir,
+                                                     path.u_name,
+                                                     unlen);
+            	if (path.d_dir == NULL) {
                 	/* path.m_name is set by adddir */
-            	    if (NULL == (path.d_dir = adddir( vol, dstack[cidx].dir, &path) ) ) {
+            	    if ((path.d_dir = dir_add(vol,
+                                              currentdir,
+                                              &path,
+                                              unlen)) == NULL) {
 						result = AFPERR_MISC;
 						goto catsearch_end;
 					}
                 }
-                path.m_name = path.d_dir->d_m_name; 
+                path.m_name = cfrombstr(path.d_dir->d_m_name);
                 	
 				if (addstack(path.u_name, path.d_dir, cidx) == -1) {
 					result = AFPERR_MISC;
 					goto catsearch_end;
 				} 
+            } else {
+            	path.d_dir = currentdir;
             }
-            else {
-            	/* yes it sucks for directory d_dir is the directory, for file it's the parent directory*/
-            	path.d_dir = dstack[cidx].dir;
-            }
+
 			ccr = crit_check(vol, &path);
 
 			/* bit 0 means that criteria has been met */
 			if ((ccr & 1)) {
-				r = rslt_add ( vol, &path, &rrbuf, ext);
+				r = rslt_add (obj, vol, &path, &rrbuf, ext);
 				
 				if (r == 0) {
 					result = AFPERR_MISC;
@@ -655,30 +687,184 @@ catsearch_pause:
 
 catsearch_end: /* Exiting catsearch: error condition */
 	*rsize = rrbuf - rbuf;
-	if (orig_dir != NULL) {
-		chdir(orig_dir);
-		free(orig_dir);
-	}
+    if (cwd != -1) {
+        if ((fchdir(cwd)) != 0) {
+            LOG(log_debug, logtype_afpd, "error chdiring back: %s", strerror(errno));        
+        }
+        close(cwd);
+    }
 	return result;
 } /* catsearch() */
 
+/*!
+ * This function performs a CNID db search
+ *
+ * Uses globals c1, c2, the search criteria
+ *
+ * @param vol       (r)  volume we are searching on ...
+ * @param dir       (rw) directory we are starting from ...
+ * @param uname     (r)  UNIX name of object to search
+ * @param rmatches  (r)  maximum number of matches we can return
+ * @param pos       (r)  position we've stopped recently
+ * @param rbuf      (w)  output buffer
+ * @param nrecs     (w)  number of matches
+ * @param rsize     (w)  length of data written to output buffer
+ * @param ext       (r)  extended search flag
+ */
+static int catsearch_db(const AFPObj *obj,
+                        struct vol *vol,
+                        struct dir *dir,  
+                        const char *uname,
+                        int rmatches,
+                        uint32_t *pos,
+                        char *rbuf,
+                        uint32_t *nrecs,
+                        int *rsize,
+                        int ext)
+{
+    static char resbuf[DBD_MAX_SRCH_RSLTS * sizeof(cnid_t)];
+    static uint32_t cur_pos;
+    static int num_matches;
+    int ccr ,r;
+	int result = AFP_OK;
+    struct path path;
+	char *rrbuf = rbuf;
+    char buffer[MAXPATHLEN +2];
+    uint16_t flags = CONV_TOLOWER;
+
+    LOG(log_debug, logtype_afpd, "catsearch_db(req pos: %u): {pos: %u, name: %s}",
+        *pos, cur_pos, uname);
+        
+	if (*pos != 0 && *pos != cur_pos) {
+		result = AFPERR_CATCHNG;
+		goto catsearch_end;
+	}
+
+    if (cur_pos == 0 || *pos == 0) {
+        if (convert_charset(vol->v_volcharset,
+                            vol->v_volcharset,
+                            vol->v_maccharset,
+                            uname,
+                            strlen(uname),
+                            buffer,
+                            MAXPATHLEN,
+                            &flags) == (size_t)-1) {
+            LOG(log_error, logtype_afpd, "catsearch_db: conversion error");
+            result = AFPERR_MISC;
+            goto catsearch_end;
+        }
+
+        LOG(log_debug, logtype_afpd, "catsearch_db: %s", buffer);
+
+        if ((num_matches = cnid_find(vol->v_cdb,
+                                     buffer,
+                                     strlen(uname),
+                                     resbuf,
+                                     sizeof(resbuf))) == -1) {
+            result = AFPERR_MISC;
+            goto catsearch_end;
+        }
+    }
+	
+	while (cur_pos < num_matches) {
+        char *name;
+        cnid_t cnid, did;
+        char resolvebuf[12 + MAXPATHLEN + 1];
+        struct dir *dir;
+
+        /* Next CNID to process from buffer */
+        memcpy(&cnid, resbuf + cur_pos * sizeof(cnid_t), sizeof(cnid_t));
+        did = cnid;
+
+        if ((name = cnid_resolve(vol->v_cdb, &did, resolvebuf, 12 + MAXPATHLEN + 1)) == NULL)
+            goto next;
+        LOG(log_debug, logtype_afpd, "catsearch_db: {pos: %u, name:%s, cnid: %u}",
+            cur_pos, name, ntohl(cnid));
+        if ((dir = dirlookup(vol, did)) == NULL)
+            goto next;
+        if (movecwd(vol, dir) < 0 )
+            goto next;
+
+        memset(&path, 0, sizeof(path));
+        path.u_name = name;
+        path.m_name = utompath(vol, name, cnid, utf8_encoding(vol->v_obj));
+
+        if (of_stat(&path) != 0) {
+            switch (errno) {
+            case EACCES:
+            case ELOOP:
+                goto next;
+            case ENOENT:
+                
+            default:
+                result = AFPERR_MISC;
+                goto catsearch_end;
+            } 
+        }
+        /* For files path.d_dir is the parent dir, for dirs its the dir itself */
+        if (S_ISDIR(path.st.st_mode))
+            if ((dir = dirlookup(vol, cnid)) == NULL)
+                goto next;
+        path.d_dir = dir;
+
+        LOG(log_maxdebug, logtype_afpd,"catsearch_db: dir: %s, cwd: %s, name: %s", 
+            cfrombstr(dir->d_fullpath), getcwdpath(), path.u_name);
+
+        /* At last we can check the search criteria */
+        ccr = crit_check(vol, &path);
+        if ((ccr & 1)) {
+            LOG(log_debug, logtype_afpd,"catsearch_db: match: %s/%s",
+                getcwdpath(), path.u_name);
+            /* bit 1 means that criteria has been met */
+            r = rslt_add(obj, vol, &path, &rrbuf, ext);
+            if (r == 0) {
+                result = AFPERR_MISC;
+                goto catsearch_end;
+            } 
+            *nrecs += r;
+            /* Number of matches limit */
+            if (--rmatches == 0) 
+                goto catsearch_pause;
+            /* Block size limit */
+            if (rrbuf - rbuf >= 448)
+                goto catsearch_pause;
+        }
+    next:
+        cur_pos++;
+    } /* while */
+
+	/* finished */
+	result = AFPERR_EOF;
+    cur_pos = 0;
+	goto catsearch_end;
+
+catsearch_pause:
+    *pos = cur_pos;
+
+catsearch_end: /* Exiting catsearch: error condition */
+	*rsize = rrbuf - rbuf;
+    LOG(log_debug, logtype_afpd, "catsearch_db(req pos: %u): {pos: %u}", *pos, cur_pos);
+	return result;
+}
+
 /* -------------------------- */
-int catsearch_afp(AFPObj *obj, char *ibuf, int ibuflen,
-                  char *rbuf, int *rbuflen, int ext)
+static int catsearch_afp(AFPObj *obj _U_, char *ibuf, size_t ibuflen,
+                  char *rbuf, size_t *rbuflen, int ext)
 {
     struct vol *vol;
-    u_int16_t   vid;
-    u_int16_t   spec_len;
-    u_int32_t   rmatches, reserved;
-    u_int32_t	catpos[4];
-    u_int32_t   pdid = 0;
+    uint16_t   vid;
+    uint16_t   spec_len;
+    uint32_t   rmatches, reserved;
+    uint32_t	catpos[4];
+    uint32_t   pdid = 0;
     int ret, rsize;
-    u_int32_t nrecs = 0;
+    uint32_t nrecs = 0;
     unsigned char *spec1, *spec2, *bspec1, *bspec2;
     size_t	len;
-    u_int16_t	namelen;
-    u_int16_t	flags;
-    char  	tmppath[256];
+    uint16_t	namelen;
+    uint16_t	flags;
+    char  	    tmppath[256];
+    char        *uname;
 
     *rbuflen = 0;
 
@@ -736,8 +922,8 @@ int catsearch_afp(AFPObj *obj, char *ibuf, int ibuflen,
     }
 
     /* Parse file specifications */
-    spec1 = ibuf;
-    spec2 = ibuf + spec_len + 2;
+    spec1 = (unsigned char*)ibuf;
+    spec2 = (unsigned char*)ibuf + spec_len + 2;
 
     spec1 += 2; 
     spec2 += 2; 
@@ -827,10 +1013,9 @@ int catsearch_afp(AFPObj *obj, char *ibuf, int ibuflen,
         /* Get the long filename */	
 		memcpy(tmppath, bspec1 + spec1[1] + 1, (bspec1 + spec1[1])[0]);
 		tmppath[(bspec1 + spec1[1])[0]]= 0;
-		len = convert_string ( vol->v_maccharset, CH_UCS2, tmppath, strlen(tmppath), c1.lname, 64);
+		len = convert_string ( vol->v_maccharset, CH_UCS2, tmppath, -1, c1.lname, sizeof(c1.lname));
         if (len == (size_t)(-1))
             return AFPERR_PARAM;
-		c1.lname[len] = 0;
 
 #if 0	
 		/* FIXME: do we need it ? It's always null ! */
@@ -850,23 +1035,33 @@ int catsearch_afp(AFPObj *obj, char *ibuf, int ibuflen,
 		/* length */
 		memcpy(&namelen, spec1, sizeof(namelen));
 		namelen = ntohs (namelen);
-		if (namelen > 255)  /* Safeguard */
-			namelen = 255;
+		if (namelen > UTF8FILELEN_EARLY)  /* Safeguard */
+			namelen = UTF8FILELEN_EARLY;
 
 		memcpy (c1.utf8name, spec1+2, namelen);
-		c1.utf8name[(namelen+1)] =0;
+		c1.utf8name[namelen] = 0;
+        if ((uname = mtoupath(vol, c1.utf8name, 0, utf8_encoding(obj))) == NULL)
+            return AFPERR_PARAM;
 
  		/* convert charset */
 		flags = CONV_PRECOMPOSE;
  		len = convert_charset(CH_UTF8_MAC, CH_UCS2, CH_UTF8, c1.utf8name, namelen, c1.utf8name, 512, &flags);
         if (len == (size_t)(-1))
             return AFPERR_PARAM;
- 		c1.utf8name[len]=0;
     }
     
     /* Call search */
     *rbuflen = 24;
-    ret = catsearch(vol, vol->v_dir, rmatches, &catpos[0], rbuf+24, &nrecs, &rsize, ext);
+    if ((c1.rbitmap & (1 << FILPBIT_PDINFO))
+        && !(c1.rbitmap & (1<<CATPBIT_PARTIAL))
+        && (strcmp(vol->v_cnidscheme, "dbd") == 0)
+        && (vol->v_flags & AFPVOL_SEARCHDB))
+        /* we've got a name and it's a dbd volume, so search CNID database */
+        ret = catsearch_db(obj, vol, vol->v_root, uname, rmatches, &catpos[0], rbuf+24, &nrecs, &rsize, ext);
+    else
+        /* perform a slow filesystem tree search */
+        ret = catsearch(obj, vol, vol->v_root, rmatches, &catpos[0], rbuf+24, &nrecs, &rsize, ext);
+
     memcpy(rbuf, catpos, sizeof(catpos));
     rbuf += sizeof(catpos);
 
@@ -887,15 +1082,15 @@ int catsearch_afp(AFPObj *obj, char *ibuf, int ibuflen,
 } /* catsearch_afp */
 
 /* -------------------------- */
-int afp_catsearch (AFPObj *obj, char *ibuf, int ibuflen,
-                  char *rbuf, int *rbuflen)
+int afp_catsearch (AFPObj *obj, char *ibuf, size_t ibuflen,
+                  char *rbuf, size_t *rbuflen)
 {
 	return catsearch_afp( obj, ibuf, ibuflen, rbuf, rbuflen, 0);
 }
 
 
-int afp_catsearch_ext (AFPObj *obj, char *ibuf, int ibuflen,
-                  char *rbuf, int *rbuflen)
+int afp_catsearch_ext (AFPObj *obj, char *ibuf, size_t ibuflen,
+                  char *rbuf, size_t *rbuflen)
 {
 	return catsearch_afp( obj, ibuf, ibuflen, rbuf, rbuflen, 1);
 }
