@@ -24,10 +24,8 @@
 #ifndef AFPD_DIRECTORY_H
 #define AFPD_DIRECTORY_H 1
 
-#include <sys/cdefs.h>
 #include <sys/types.h>
-/*#include <sys/stat.h>*/ /* including it here causes some confusion */
-#include <netatalk/endian.h>
+#include <arpa/inet.h>
 #include <dirent.h>
 
 /* sys/types.h usually snarfs in major/minor macros. if they don't
@@ -93,7 +91,7 @@ struct maccess {
 #define	AR_UWRITE	(1<<2)
 #define	AR_UOWN		(1<<7)
 
-extern q_t *invalid_dircache_entries;
+q_t *invalid_dircache_entries;
 
 typedef int (*dir_loop)(struct dirent *, char *, void *);
 
@@ -113,21 +111,19 @@ extern struct path *cname (struct vol *, struct dir *, char **);
 
 extern int         deletecurdir (struct vol *);
 extern mode_t      mtoumode (struct maccess *);
-extern void        utommode (struct stat *, struct maccess *);
-extern int         getdirparams (const struct vol *, u_int16_t, struct path *,
+extern int         getdirparams (const AFPObj *obj, const struct vol *, uint16_t, struct path *,
                                  struct dir *, char *, size_t *);
 
-extern int         setdirparams(struct vol *, struct path *, u_int16_t, char *);
+extern int         setdirparams(struct vol *, struct path *, uint16_t, char *);
 extern int         renamedir(const struct vol *, int, char *, char *, struct dir *,
                              struct dir *, char *);
 extern int         path_error(struct path *, int error);
-extern void        setdiroffcnt(struct dir *dir, struct stat *st,  u_int32_t count);
+extern void        setdiroffcnt(struct dir *dir, struct stat *st,  uint32_t count);
 extern int         dirreenumerate(struct dir *dir, struct stat *st);
 extern int         for_each_dirent(const struct vol *, char *, dir_loop , void *);
-extern int         check_access(char *name , int mode);
-extern int         file_access(struct path *path, int mode);
+extern int         check_access(const AFPObj *obj, struct vol *, char *name , int mode);
+extern int         file_access(const AFPObj *obj, struct vol *vol, struct path *path, int mode);
 extern int         netatalk_unlink (const char *name);
-extern int         caseenumerate (const struct vol *, struct path *, struct dir *);
 
 /* from enumerate.c */
 extern char        *check_dirent (const struct vol *, char *);
