@@ -37,8 +37,6 @@
 #include <arpa/inet.h>
 #include <netdb.h>
 
-#include <netatalk/at.h>
-
 #include <atalk/adouble.h>
 #include <atalk/vfs.h>
 #include <atalk/logger.h>
@@ -85,7 +83,7 @@ static struct fce_close_event last_close_event;
 /*
  *
  * Initialize network structs for any listeners
- * We don't give return code because all errors are handled internally (I hope..)
+ * We dont give return code because all errors are handled internally (I hope..)
  *
  * */
 void fce_init_udp()
@@ -231,7 +229,7 @@ static void pack_fce_packet(struct fce_packet *packet, unsigned char *buf, int m
 
 /*
  * Send the fce information to all (connected) listeners
- * We don't give return code because all errors are handled internally (I hope..)
+ * We dont give return code because all errors are handled internally (I hope..)
  * */
 static void send_fce_event( char *path, int mode )
 {    
@@ -309,7 +307,7 @@ static void send_fce_event( char *path, int mode )
         /* Problems ? */
         if (sent_data != data_len) {
             /* Argh, socket broke, we close and retry later */
-            LOG(log_error, logtype_afpd, "send_fce_event: error sending packet to %s:%s, transferred %d of %d: %s",
+            LOG(log_error, logtype_afpd, "send_fce_event: error sending packet to %s:%s, transfered %d of %d: %s",
                 udp_entry->addr, udp_entry->port, sent_data, data_len, strerror(errno));
 
             close( udp_entry->sock );
@@ -515,24 +513,15 @@ int fce_register_new_file( struct path *path )
 
 int fce_register_file_modification( struct ofork *ofork )
 {
-    char *u_name = NULL;
-    struct vol *vol;
     int ret = AFP_OK;
 
-    if (ofork == NULL || ofork->of_vol == NULL)
+    if (ofork == NULL)
         return AFPERR_PARAM;
 
     if (!(fce_ev_enabled & (1 << FCE_FILE_MODIFY)))
         return ret;
 
-    vol = ofork->of_vol;
-
-    if (NULL == (u_name = mtoupath(vol, of_name(ofork), ofork->of_did, utf8_encoding()))) 
-    {
-        return AFPERR_MISC;
-    }
-    
-    ret = register_fce( u_name, false, FCE_FILE_MODIFY );
+    ret = register_fce(of_name(ofork), false, FCE_FILE_MODIFY );
     
     return ret;    
 }
@@ -586,7 +575,7 @@ int fce_set_events(const char *events)
 
     fce_ev_enabled = 0;
 
-    for (p = strtok(e, ","); p; p = strtok(NULL, ",")) {
+    for (p = strtok(e, ", "); p; p = strtok(NULL, ", ")) {
         if (strcmp(p, "fmod") == 0) {
             fce_ev_enabled |= (1 << FCE_FILE_MODIFY);
         } else if (strcmp(p, "fdel") == 0) {
