@@ -11,10 +11,8 @@
 #include <stdlib.h>
 #include <string.h>
 #include <signal.h>
-
 #include <sys/param.h>
 #include <sys/uio.h>
-#include <atalk/logger.h>
 #include <sys/time.h>
 #include <sys/socket.h>
 #include <sys/poll.h>
@@ -22,19 +20,18 @@
 #include <sys/wait.h>
 #include <sys/resource.h>
 
+#include <atalk/logger.h>
 #include <atalk/adouble.h>
-
-#include <netatalk/at.h>
 #include <atalk/compat.h>
 #include <atalk/dsi.h>
-#include <atalk/atp.h>
-#include <atalk/asp.h>
 #include <atalk/afp.h>
 #include <atalk/paths.h>
 #include <atalk/util.h>
 #include <atalk/server_child.h>
 #include <atalk/server_ipc.h>
+#include <atalk/errchk.h>
 #include <atalk/globals.h>
+#include <atalk/netatalk_conf.h>
 
 #include "afp_config.h"
 #include "status.h"
@@ -42,21 +39,15 @@
 #include "uam_auth.h"
 #include "afp_zeroconf.h"
 
-#ifdef TRU64
-#include <sys/security.h>
-#include <prot.h>
-#include <sia.h>
+#define AFP_LISTENERS 32
+#define FDSET_SAFETY  5
 
-static int argc = 0;
-static char **argv = NULL;
-#endif /* TRU64 */
+unsigned char nologin = 0;
 
-unsigned char	nologin = 0;
-
-struct afp_options default_options;
-static AFPConfig *configs;
+static AFPObj obj;
 static server_child *server_children;
 static sig_atomic_t reloadconfig = 0;
+static sig_atomic_t gotsigchld = 0;
 
 /* Two pointers to dynamic allocated arrays which store pollfds and associated data */
 static struct pollfd *fdset;
@@ -65,59 +56,58 @@ static int fdset_size;          /* current allocated size */
 static int fdset_used;          /* number of used elements */
 static int disasociated_ipc_fd; /* disasociated sessions uses this fd for IPC */
 
-#ifdef TRU64
-void afp_get_cmdline( int *ac, char ***av)
-{
-    *ac = argc;
-    *av = argv;
-}
-#endif /* TRU64 */
+static afp_child_t *dsi_start(AFPObj *obj, DSI *dsi, server_child *server_children);
 
-/* This is registered with atexit() */
-static void afp_exit(void)
+static void afp_exit(int ret)
 {
-    if (parent_or_child == 0)
-        /* Only do this in the parent */
-        server_unlock(default_options.pidfile);
+    exit(ret);
 }
 
 
 /* ------------------
    initialize fd set we are waiting for.
 */
-static void fd_set_listening_sockets(void)
+static void fd_set_listening_sockets(const AFPObj *config)
 {
-    AFPConfig   *config;
+    DSI *dsi;
 
-    for (config = configs; config; config = config->next) {
-        if (config->fd < 0) /* for proxies */
-            continue;
-        fdset_add_fd(&fdset, &polldata, &fdset_used, &fdset_size, config->fd, LISTEN_FD, config);
+    for (dsi = config->dsi; dsi; dsi = dsi->next) {
+        fdset_add_fd(config->options.connections + AFP_LISTENERS + FDSET_SAFETY,
+                     &fdset,
+                     &polldata,
+                     &fdset_used,
+                     &fdset_size,
+                     dsi->serversock,
+                     LISTEN_FD,
+                     dsi);
     }
-    fdset_add_fd(&fdset, &polldata, &fdset_used, &fdset_size, disasociated_ipc_fd, DISASOCIATED_IPC_FD, NULL);
+
+    if (config->options.flags & OPTION_KEEPSESSIONS)
+        fdset_add_fd(config->options.connections + AFP_LISTENERS + FDSET_SAFETY,
+                     &fdset,
+                     &polldata,
+                     &fdset_used,
+                     &fdset_size,
+                     disasociated_ipc_fd,
+                     DISASOCIATED_IPC_FD,
+                     NULL);
 }
  
-static void fd_reset_listening_sockets(void)
+static void fd_reset_listening_sockets(const AFPObj *config)
 {
-    AFPConfig   *config;
+    const DSI *dsi;
 
-    for (config = configs; config; config = config->next) {
-        if (config->fd < 0) /* for proxies */
-            continue;
-        fdset_del_fd(&fdset, &polldata, &fdset_used, &fdset_size, config->fd);
+    for (dsi = config->dsi; dsi; dsi = dsi->next) {
+        fdset_del_fd(&fdset, &polldata, &fdset_used, &fdset_size, dsi->serversock);
     }
-    fdset_del_fd(&fdset, &polldata, &fdset_used, &fdset_size, disasociated_ipc_fd);
+
+    if (config->options.flags & OPTION_KEEPSESSIONS)
+        fdset_del_fd(&fdset, &polldata, &fdset_used, &fdset_size, disasociated_ipc_fd);
 }
 
 /* ------------------ */
 static void afp_goaway(int sig)
 {
-        AFPConfig *config;
-
-#ifndef NO_DDP
-    asp_kill(sig);
-#endif /* ! NO_DDP */
-
     switch( sig ) {
 
     case SIGTERM:
@@ -127,17 +117,18 @@ static void afp_goaway(int sig)
             LOG(log_note, logtype_afpd, "AFP Server shutting down on SIGTERM");
             break;
         case SIGQUIT:
-            LOG(log_note, logtype_afpd, "AFP Server shutting down on SIGQUIT, NOT disconnecting clients");
+            if (obj.options.flags & OPTION_KEEPSESSIONS) {
+                LOG(log_note, logtype_afpd, "AFP Server shutting down on SIGQUIT, NOT disconnecting clients");
+            } else {
+                LOG(log_note, logtype_afpd, "AFP Server shutting down on SIGQUIT");
+                sig = SIGTERM;
+            }
             break;
         }
         if (server_children)
             server_child_kill(server_children, CHILD_DSIFORK, sig);
 
-        for (config = configs; config; config = config->next)
-            if (config->server_cleanup)
-                config->server_cleanup(config);
-        server_unlock(default_options.pidfile);
-        exit(0);
+        _exit(0);
         break;
 
     case SIGUSR1 :
@@ -154,13 +145,18 @@ static void afp_goaway(int sig)
         reloadconfig = 1;
         break;
 
+    case SIGCHLD:
+        /* w/ a configuration file, we can force a re-read if we want */
+        gotsigchld = 1;
+        break;
+
     default :
         LOG(log_error, logtype_afpd, "afp_goaway: bad signal" );
     }
     return;
 }
 
-static void child_handler(int sig _U_)
+static void child_handler(void)
 {
     int fd;
     int status, i;
@@ -197,16 +193,16 @@ static int setlimits(void)
     struct rlimit rlim;
 
     if (getrlimit(RLIMIT_NOFILE, &rlim) != 0) {
-        LOG(log_error, logtype_afpd, "setlimits: %s", strerror(errno));
-        exit(1);
+        LOG(log_warning, logtype_afpd, "setlimits: reading current limits failed: %s", strerror(errno));
+        return -1;
     }
     if (rlim.rlim_cur != RLIM_INFINITY && rlim.rlim_cur < 65535) {
         rlim.rlim_cur = 65535;
         if (rlim.rlim_max != RLIM_INFINITY && rlim.rlim_max < 65535)
             rlim.rlim_max = 65535;
         if (setrlimit(RLIMIT_NOFILE, &rlim) != 0) {
-            LOG(log_error, logtype_afpd, "setlimits: %s", strerror(errno));
-            exit(1);
+            LOG(log_warning, logtype_afpd, "setlimits: increasing limits failed: %s", strerror(errno));
+            return -1;
         }
     }
     return 0;
@@ -214,52 +210,37 @@ static int setlimits(void)
 
 int main(int ac, char **av)
 {
-    AFPConfig           *config;
     fd_set              rfds;
     void                *ipc;
     struct sigaction	sv;
     sigset_t            sigs;
     int                 ret;
 
-#ifdef TRU64
-    argc = ac;
-    argv = av;
-    set_auth_parameters( ac, av );
-#endif /* TRU64 */
+    /* Parse argv args and initialize default options */
+    afp_options_parse_cmdline(&obj, ac, av);
+
+    if (!(obj.cmdlineflags & OPTION_DEBUG) && (daemonize(0, 0) != 0))
+        exit(EXITERR_SYS);
 
     /* Log SIGBUS/SIGSEGV SBT */
     fault_setup(NULL);
 
-    /* Default log setup: log to syslog */
-    setuplog("default log_note");
+    if (afp_config_parse(&obj, "afpd") != 0)
+        afp_exit(EXITERR_CONF);
 
-    afp_options_init(&default_options);
-    if (!afp_options_parse(ac, av, &default_options))
-        exit(EXITERR_CONF);
-
-    /* Save the user's current umask for use with CNID (and maybe some 
-     * other things, too). */
-    default_options.save_mask = umask( default_options.umask );
-
-    switch(server_lock("afpd", default_options.pidfile,
-                       default_options.flags & OPTION_DEBUG)) {
-    case -1: /* error */
-        exit(EXITERR_SYS);
-    case 0: /* child */
-        break;
-    default: /* server */
-        exit(0);
-    }
-    atexit(afp_exit);
+    /* Save the user's current umask */
+    obj.options.save_mask = umask(obj.options.umask);
 
     /* install child handler for asp and dsi. we do this before afp_goaway
      * as afp_goaway references stuff from here. 
      * XXX: this should really be setup after the initial connections. */
-    if (!(server_children = server_child_alloc(default_options.connections,
-                            CHILD_NFORKS))) {
+    if (!(server_children = server_child_alloc(obj.options.connections, CHILD_NFORKS))) {
         LOG(log_error, logtype_afpd, "main: server_child alloc: %s", strerror(errno) );
-        exit(EXITERR_SYS);
+        afp_exit(EXITERR_SYS);
     }
+    
+    sigemptyset(&sigs);
+    pthread_sigmask(SIG_SETMASK, &sigs, NULL);
 
     memset(&sv, 0, sizeof(sv));    
     /* linux at least up to 2.4.22 send a SIGXFZ for vfat fs,
@@ -269,11 +250,12 @@ int main(int ac, char **av)
     sigemptyset( &sv.sa_mask );
     if (sigaction(SIGXFSZ, &sv, NULL ) < 0 ) {
         LOG(log_error, logtype_afpd, "main: sigaction: %s", strerror(errno) );
-        exit(EXITERR_SYS);
+        afp_exit(EXITERR_SYS);
     }
 #endif
     
-    sv.sa_handler = child_handler;
+    sv.sa_handler = afp_goaway; /* handler for all sigs */
+
     sigemptyset( &sv.sa_mask );
     sigaddset(&sv.sa_mask, SIGALRM);
     sigaddset(&sv.sa_mask, SIGHUP);
@@ -283,10 +265,9 @@ int main(int ac, char **av)
     sv.sa_flags = SA_RESTART;
     if ( sigaction( SIGCHLD, &sv, NULL ) < 0 ) {
         LOG(log_error, logtype_afpd, "main: sigaction: %s", strerror(errno) );
-        exit(EXITERR_SYS);
+        afp_exit(EXITERR_SYS);
     }
 
-    sv.sa_handler = afp_goaway;
     sigemptyset( &sv.sa_mask );
     sigaddset(&sv.sa_mask, SIGALRM);
     sigaddset(&sv.sa_mask, SIGTERM);
@@ -296,7 +277,7 @@ int main(int ac, char **av)
     sv.sa_flags = SA_RESTART;
     if ( sigaction( SIGUSR1, &sv, NULL ) < 0 ) {
         LOG(log_error, logtype_afpd, "main: sigaction: %s", strerror(errno) );
-        exit(EXITERR_SYS);
+        afp_exit(EXITERR_SYS);
     }
 
     sigemptyset( &sv.sa_mask );
@@ -308,10 +289,9 @@ int main(int ac, char **av)
     sv.sa_flags = SA_RESTART;
     if ( sigaction( SIGHUP, &sv, NULL ) < 0 ) {
         LOG(log_error, logtype_afpd, "main: sigaction: %s", strerror(errno) );
-        exit(EXITERR_SYS);
+        afp_exit(EXITERR_SYS);
     }
 
-
     sigemptyset( &sv.sa_mask );
     sigaddset(&sv.sa_mask, SIGALRM);
     sigaddset(&sv.sa_mask, SIGHUP);
@@ -321,7 +301,7 @@ int main(int ac, char **av)
     sv.sa_flags = SA_RESTART;
     if ( sigaction( SIGTERM, &sv, NULL ) < 0 ) {
         LOG(log_error, logtype_afpd, "main: sigaction: %s", strerror(errno) );
-        exit(EXITERR_SYS);
+        afp_exit(EXITERR_SYS);
     }
 
     sigemptyset( &sv.sa_mask );
@@ -333,10 +313,10 @@ int main(int ac, char **av)
     sv.sa_flags = SA_RESTART;
     if (sigaction(SIGQUIT, &sv, NULL ) < 0 ) {
         LOG(log_error, logtype_afpd, "main: sigaction: %s", strerror(errno) );
-        exit(EXITERR_SYS);
+        afp_exit(EXITERR_SYS);
     }
 
-    /* afpd.conf: not in config file: lockfile, connections, configfile
+    /* afp.conf:  not in config file: lockfile, configfile
      *            preference: command-line provides defaults.
      *                        config file over-writes defaults.
      *
@@ -355,25 +335,31 @@ int main(int ac, char **av)
     sigaddset(&sigs, SIGCHLD);
 
     pthread_sigmask(SIG_BLOCK, &sigs, NULL);
-    if (!(configs = configinit(&default_options))) {
+    if (configinit(&obj) != 0) {
         LOG(log_error, logtype_afpd, "main: no servers configured");
-        exit(EXITERR_CONF);
+        afp_exit(EXITERR_CONF);
     }
     pthread_sigmask(SIG_UNBLOCK, &sigs, NULL);
 
-    /* Register CNID  */
+    /* Initialize */
     cnid_init();
-
+    
     /* watch atp, dsi sockets and ipc parent/child file descriptor. */
-    disasociated_ipc_fd = ipc_server_uds(_PATH_AFP_IPC);
-    fd_set_listening_sockets();
+
+    if (obj.options.flags & OPTION_KEEPSESSIONS) {
+        LOG(log_note, logtype_afpd, "Activating continous service");
+        disasociated_ipc_fd = ipc_server_uds(_PATH_AFP_IPC);
+    }
+
+    fd_set_listening_sockets(&obj);
 
     /* set limits */
     (void)setlimits();
 
     afp_child_t *child;
-    int fd[2];  /* we only use one, but server_child_add expects [2] */
+    int recon_ipc_fd;
     pid_t pid;
+    int saveerrno;
 
     /* wait for an appleshare connection. parent remains in the loop
      * while the children get handled by afp_over_{asp,dsi}.  this is
@@ -386,27 +372,28 @@ int main(int ac, char **av)
         pthread_sigmask(SIG_UNBLOCK, &sigs, NULL);
         ret = poll(fdset, fdset_used, -1);
         pthread_sigmask(SIG_BLOCK, &sigs, NULL);
-        int saveerrno = errno;
+        saveerrno = errno;
+
+        if (gotsigchld) {
+            gotsigchld = 0;
+            child_handler();
+            continue;
+        }
 
         if (reloadconfig) {
             nologin++;
             auth_unload();
-            fd_reset_listening_sockets();
+            fd_reset_listening_sockets(&obj);
 
             LOG(log_info, logtype_afpd, "re-reading configuration file");
-            for (config = configs; config; config = config->next)
-                if (config->server_cleanup)
-                    config->server_cleanup(config);
 
-            /* configfree close atp socket used for DDP tickle, there's an issue
-             * with atp tid. */
-            configfree(configs, NULL);
-            if (!(configs = configinit(&default_options))) {
+            configfree(&obj, NULL);
+            if (configinit(&obj) != 0) {
                 LOG(log_error, logtype_afpd, "config re-read: no servers configured");
-                exit(EXITERR_CONF);
+                afp_exit(EXITERR_CONF);
             }
 
-            fd_set_listening_sockets();
+            fd_set_listening_sockets(&obj);
 
             nologin = 0;
             reloadconfig = 0;
@@ -425,15 +412,20 @@ int main(int ac, char **av)
         }
 
         for (int i = 0; i < fdset_used; i++) {
-            if (fdset[i].revents & (POLLIN | POLLERR | POLLHUP)) {
+            if (fdset[i].revents & (POLLIN | POLLERR | POLLHUP | POLLNVAL)) {
                 switch (polldata[i].fdtype) {
 
                 case LISTEN_FD:
-                    config = (AFPConfig *)polldata[i].data;
-                    /* config->server_start is afp_config.c:dsi_start() for DSI */
-                    if (child = config->server_start(config, configs, server_children)) {
+                    if (child = dsi_start(&obj, (DSI *)polldata[i].data, server_children)) {
                         /* Add IPC fd to select fd set */
-                        fdset_add_fd(&fdset, &polldata, &fdset_used, &fdset_size, child->ipc_fds[0], IPC_FD, child);
+                        fdset_add_fd(obj.options.connections + AFP_LISTENERS + FDSET_SAFETY,
+                                     &fdset,
+                                     &polldata,
+                                     &fdset_used,
+                                     &fdset_size,
+                                     child->ipc_fd,
+                                     IPC_FD,
+                                     child);
                     }
                     break;
 
@@ -441,11 +433,11 @@ int main(int ac, char **av)
                     child = (afp_child_t *)polldata[i].data;
                     LOG(log_debug, logtype_afpd, "main: IPC request from child[%u]", child->pid);
 
-                    if ((ret = ipc_server_read(server_children, child->ipc_fds[0])) == 0) {
-                        fdset_del_fd(&fdset, &polldata, &fdset_used, &fdset_size, child->ipc_fds[0]);
-                        close(child->ipc_fds[0]);
-                        child->ipc_fds[0] = -1;
-                        if (child->disasociated) {
+                    if (ipc_server_read(server_children, child->ipc_fd) != 0) {
+                        fdset_del_fd(&fdset, &polldata, &fdset_used, &fdset_size, child->ipc_fd);
+                        close(child->ipc_fd);
+                        child->ipc_fd = -1;
+                        if ((obj.options.flags & OPTION_KEEPSESSIONS) && child->disasociated) {
                             LOG(log_note, logtype_afpd, "main: removing reattached child[%u]", child->pid);
                             server_child_remove(server_children, CHILD_DSIFORK, child->pid);
                         }
@@ -454,22 +446,31 @@ int main(int ac, char **av)
 
                 case DISASOCIATED_IPC_FD:
                     LOG(log_debug, logtype_afpd, "main: IPC reconnect request");
-                    if ((fd[0] = accept(disasociated_ipc_fd, NULL, NULL)) == -1) {
+                    if ((recon_ipc_fd = accept(disasociated_ipc_fd, NULL, NULL)) == -1) {
                         LOG(log_error, logtype_afpd, "main: accept: %s", strerror(errno));
                         break;
                     }
-                    if (readt(fd[0], &pid, sizeof(pid_t), 0, 1) != sizeof(pid_t)) {
+                    if (readt(recon_ipc_fd, &pid, sizeof(pid_t), 0, 1) != sizeof(pid_t)) {
                         LOG(log_error, logtype_afpd, "main: readt: %s", strerror(errno));
-                        close(fd[0]);
+                        close(recon_ipc_fd);
+                        break;
                     }
-                    LOG(log_note, logtype_afpd, "main: IPC reconnect from [%u]", pid);
-                    if ((child = server_child_add(server_children, CHILD_DSIFORK, pid, fd)) == NULL) {
+                    LOG(log_note, logtype_afpd, "main: IPC reconnect from pid [%u]", pid);
+
+                    if ((child = server_child_add(server_children, CHILD_DSIFORK, pid, recon_ipc_fd)) == NULL) {
                         LOG(log_error, logtype_afpd, "main: server_child_add");
-                        close(fd[0]);
+                        close(recon_ipc_fd);
                         break;
                     }
                     child->disasociated = 1;
-                    fdset_add_fd(&fdset, &polldata, &fdset_used, &fdset_size, fd[0], IPC_FD, child);
+                    fdset_add_fd(obj.options.connections + AFP_LISTENERS + FDSET_SAFETY,
+                                 &fdset,
+                                 &polldata,
+                                 &fdset_used,
+                                 &fdset_size,
+                                 recon_ipc_fd,
+                                 IPC_FD,
+                                 child);
                     break;
 
                 default:
@@ -482,3 +483,22 @@ int main(int ac, char **av)
 
     return 0;
 }
+
+static afp_child_t *dsi_start(AFPObj *obj, DSI *dsi, server_child *server_children)
+{
+    afp_child_t *child = NULL;
+
+    if (dsi_getsession(dsi, server_children, obj->options.tickleval, &child) != 0) {
+        LOG(log_error, logtype_afpd, "dsi_start: session error: %s", strerror(errno));
+        return NULL;
+    }
+
+    /* we've forked. */
+    if (child == NULL) {
+        configfree(obj, dsi);
+        afp_over_dsi(obj); /* start a session */
+        exit (0);
+    }
+
+    return child;
+}
