@@ -1,6 +1,4 @@
 /*
- * $Id: messages.c,v 1.16.6.1.2.7 2004/02/14 00:30:50 didg Exp $
- *
  * Copyright (c) 1997 Adrian Sun (asun@zoology.washington.edu)
  * All Rights Reserved.  See COPYRIGHT.
  */
@@ -9,9 +7,7 @@
 #include "config.h"
 #endif /* HAVE_CONFIG_H */
 
-#ifdef HAVE_UNISTD_H
 #include <unistd.h>
-#endif /* HAVE_UNISTD_H */
 #include <stdio.h>
 #include <string.h>
 #include <errno.h>
@@ -19,10 +15,11 @@
 #include <atalk/afp.h>
 #include <atalk/dsi.h>
 #include <atalk/util.h>
+#include <atalk/unix.h>
 #include <atalk/logger.h>
-#include "globals.h"
-#include "misc.h"
+#include <atalk/globals.h>
 
+#include "misc.h"
 
 #define MAXMESGSIZE 199
 
@@ -35,8 +32,7 @@ void setmessage(const char *message)
     strlcpy(servermesg, message, MAXMESGSIZE);
 }
 
-void readmessage(obj)
-AFPObj *obj;
+void readmessage(AFPObj *obj)
 {
     /* Read server message from file defined as SERVERTEXT */
 #ifdef SERVERTEXT
@@ -45,10 +41,9 @@ AFPObj *obj;
     unsigned int i; 
     int rc;
     static int c;
-    uid_t euid;
-    u_int32_t maxmsgsize;
+    uint32_t maxmsgsize;
 
-    maxmsgsize = (obj->proto == AFPPROTO_DSI)?MIN(MAX(((DSI*)obj->handle)->attn_quantum, MAXMESGSIZE),MAXPATHLEN):MAXMESGSIZE;
+    maxmsgsize = MIN(MAX(obj->dsi->attn_quantum, MAXMESGSIZE), MAXPATHLEN);
 
     i=0;
     /* Construct file name SERVERTEXT/message.[pid] */
@@ -60,12 +55,12 @@ AFPObj *obj;
     sprintf(filename, "%s/message.%d", SERVERTEXT, getpid());
 
 #ifdef DEBUG
-    LOG(log_debug, logtype_afpd, "Reading file %s ", filename);
-#endif /* DEBUG */
+    LOG(log_debug9, logtype_afpd, "Reading file %s ", filename);
+#endif 
 
     message=fopen(filename, "r");
     if (message==NULL) {
-        LOG(log_info, logtype_afpd, "Unable to open file %s", filename);
+        /* try without the process id */
         sprintf(filename, "%s/message", SERVERTEXT);
         message=fopen(filename, "r");
     }
@@ -82,62 +77,59 @@ AFPObj *obj;
         /* cleanup */
         fclose(message);
 
-        /* Save effective uid and switch to root to delete file. */
-        /* Delete will probably fail otherwise, but let's try anyways */
-        euid = geteuid();
-        if (seteuid(0) < 0) {
-            LOG(log_error, logtype_afpd, "Could not switch back to root: %s",
-				strerror(errno));
-        }
+        become_root();
 
-        if ( 0 < (rc = unlink(filename)) )
+        if ((rc = unlink(filename)) != 0)
 	    LOG(log_error, logtype_afpd, "File '%s' could not be deleted", strerror(errno));
 
-        /* Drop privs again, failing this is very bad */
-        if (seteuid(euid) < 0) {
-            LOG(log_error, logtype_afpd, "Could not switch back to uid %d: %s", euid, strerror(errno));
-        }
+        unbecome_root();
 
         if (rc < 0) {
             LOG(log_error, logtype_afpd, "Error deleting %s: %s", filename, strerror(rc));
         }
 #ifdef DEBUG
         else {
-            LOG(log_info, logtype_afpd, "Deleted %s", filename);
+            LOG(log_debug9, logtype_afpd, "Deleted %s", filename);
         }
 
-        LOG(log_info, logtype_afpd, "Set server message to \"%s\"", servermesg);
-#endif /* DEBUG */
+        LOG(log_debug9, logtype_afpd, "Set server message to \"%s\"", servermesg);
+#endif
     }
     free(filename);
 #endif /* SERVERTEXT */
 }
 
-int afp_getsrvrmesg(obj, ibuf, ibuflen, rbuf, rbuflen)
-AFPObj *obj;
-char *ibuf, *rbuf;
-int ibuflen, *rbuflen;
+int afp_getsrvrmesg(AFPObj *obj, char *ibuf, size_t ibuflen _U_, char *rbuf, size_t *rbuflen)
 {
     char *message;
-    u_int16_t type, bitmap;
-    u_int16_t msgsize;
+    uint16_t type, bitmap;
+    uint16_t msgsize;
     size_t outlen = 0;
     size_t msglen = 0;
     int utf8 = 0;
 
     *rbuflen = 0;
 
-    msgsize = (obj->proto == AFPPROTO_DSI)?MAX(((DSI*)obj->handle)->attn_quantum, MAXMESGSIZE):MAXMESGSIZE;
+    msgsize = MAX(obj->dsi->attn_quantum, MAXMESGSIZE);
 
     memcpy(&type, ibuf + 2, sizeof(type));
     memcpy(&bitmap, ibuf + 4, sizeof(bitmap));
 
+    message = servermesg;
     switch (ntohs(type)) {
     case AFPMESG_LOGIN: /* login */
-        message = obj->options.loginmesg;
+        /* at least TIGER loses server messages
+         * if it receives a server msg attention before
+         * it has asked the login msg...
+         * Workaround: concatenate the two if any, ugly.
+         */
+        if (obj->options.loginmesg) {
+            if (*message)
+                strlcat(message, " - ", MAXMESGSIZE);
+            strlcat(message, obj->options.loginmesg, MAXMESGSIZE);
+        }
         break;
     case AFPMESG_SERVER: /* server */
-        message = servermesg;
         break;
     default:
         return AFPERR_BITMAP;
@@ -185,6 +177,6 @@ int ibuflen, *rbuflen;
 	*rbuflen += 1;
     }
     *rbuflen += outlen;
-
+    *message = 0;
     return AFP_OK;
 }
