@@ -15,56 +15,31 @@
 #include <string.h>
 #include <errno.h>
 #include <sys/types.h>
-/* STDC check */
-#if STDC_HEADERS
 #include <string.h>
-#else /* STDC_HEADERS */
-#ifndef HAVE_STRCHR
-#define strchr index
-#define strrchr index
-#endif /* HAVE_STRCHR */
-char *strchr (), *strrchr ();
-#ifndef HAVE_MEMCPY
-#define memcpy(d,s,n) bcopy ((s), (d), (n))
-#define memmove(d,s,n) bcopy ((s), (d), (n))
-#endif /* ! HAVE_MEMCPY */
-#endif /* STDC_HEADERS */
 #include <sys/stat.h>
 #include <sys/time.h>
 #include <sys/param.h>
-#ifdef HAVE_UNISTD_H
 #include <unistd.h>
-#endif /* HAVE_UNISTD_H */
-#ifdef HAVE_FCNTL_H
 #include <fcntl.h>
-#endif /* HAVE_FCNTL_H */
-
-#if defined(HAVE_SYS_MNTTAB_H) || defined(__svr4__)
-#include <sys/mntent.h>
-#endif
 
 #include <atalk/logger.h>
 #include <atalk/afp.h>
 #include <atalk/compat.h>
-#include <atalk/util.h>
+#include <atalk/unix.h>
 
 #include "auth.h"
 #include "volume.h"
 #include "unix.h"
 
 #ifdef HAVE_LIBQUOTA
-#include <quota.h>
-
+#include <quota/quota.h>
 
 static int
-getfreespace(struct vol *vol, VolSpace *bfree, VolSpace *btotal,
-    id_t id, int idtype)
+getfreespace(const AFPObj *obj, struct vol *vol, VolSpace *bfree, VolSpace *btotal,
+	     uid_t uid, const char *classq)
 {
-	uid_t prevuid;
-	const char *msg;
-	struct quotahandle *qh;
-	struct quotakey qk;
-	struct quotaval qv;
+	int retq;
+	struct ufs_quota_entry ufsq[QUOTA_NLIMITS];
 	time_t now;
 
 	if (time(&now) == -1) {
@@ -73,107 +48,64 @@ getfreespace(struct vol *vol, VolSpace *bfree, VolSpace *btotal,
 		return -1;
 	}
 
-	prevuid = geteuid();
-	if (prevuid == -1) {
-		LOG(log_info, logtype_afpd, "geteuid(): %s",
-		    strerror(errno));
-		return -1;
+    become_root();
+
+	if ((retq = getfsquota(obj, vol, ufsq, uid, classq)) < 0) {
+		LOG(log_info, logtype_afpd, "getfsquota(%s, %s): %s",
+		    vol->v_path, classq, strerror(errno));
 	}
 
-	if ( seteuid( getuid() ) != 0 )  {
-		LOG(log_info, logtype_afpd, "seteuid(): %s",
-		    strerror(errno));
-		return -1;
-	}
+    unbecome_root();
 
-	/*
-	 * In a tidier world we might keep the quotahandle open for longer...
-	 */
-	qh = quota_open(vol->v_path);
-	if (qh == NULL) {
-		if (errno == EOPNOTSUPP || errno == ENXIO) {
-			/* no quotas on this volume */
-			seteuid( prevuid );
-			return 0;
-		}
-	   
-		LOG(log_info, logtype_afpd, "quota_open(%s): %s", vol->v_path,
-		    strerror(errno));
-		seteuid( prevuid );
-		return -1;
-	}
+	if (retq < 1)
+		return retq;
 
-	qk.qk_idtype = idtype;
-	qk.qk_id = id;
-	qk.qk_objtype = QUOTA_OBJTYPE_BLOCKS;
-	if (quota_get(qh, &qk, &qv) < 0) {
-		if (errno == ENOENT) {
-			/* no quotas for this id */
-			quota_close(qh);
-			seteuid( prevuid );
-			return 0;
-		}
-		msg = strerror(errno);
-		LOG(log_info, logtype_afpd, "quota_get(%s, %s): %s",
-		    vol->v_path, quota_idtype_getname(qh, idtype), msg);
-		quota_close(qh);
-		seteuid( prevuid );
-		return -1;
-	}
-
-	quota_close(qh);
-
-        seteuid( prevuid );
-
-	if (qv.qv_usage >= qv.qv_hardlimit ||
-	    (qv.qv_usage >= qv.qv_softlimit && now > qv.qv_expiretime)) {
+	switch(QL_STATUS(quota_check_limit(ufsq[QUOTA_LIMIT_BLOCK].ufsqe_cur, 1,
+	    ufsq[QUOTA_LIMIT_BLOCK].ufsqe_softlimit,
+	    ufsq[QUOTA_LIMIT_BLOCK].ufsqe_hardlimit,
+	    ufsq[QUOTA_LIMIT_BLOCK].ufsqe_time, now))) {
+	case QL_S_DENY_HARD:
+	case QL_S_DENY_GRACE:
 		*bfree = 0;
-		*btotal = dbtob(qv.qv_usage);
+		*btotal = dbtob(ufsq[QUOTA_LIMIT_BLOCK].ufsqe_cur);
+		break;
+	default:
+		*bfree = dbtob(ufsq[QUOTA_LIMIT_BLOCK].ufsqe_hardlimit -
+		    ufsq[QUOTA_LIMIT_BLOCK].ufsqe_cur);
+		*btotal = dbtob(ufsq[QUOTA_LIMIT_BLOCK].ufsqe_hardlimit);
+		break;
 	}
-	else {
-		*bfree = dbtob(qv.qv_hardlimit - qv.qv_usage);
-		*btotal = dbtob(qv.qv_hardlimit);
-	}
-
 	return 1;
 }
 
-int uquota_getvolspace( struct vol *vol, VolSpace *bfree, VolSpace *btotal, const u_int32_t bsize)
+int uquota_getvolspace(const AFPObj *obj, struct vol *vol, VolSpace *bfree, VolSpace *btotal, const u_int32_t bsize)
 {
-	int uret, gret;
+	int uretq, gretq;
 	VolSpace ubfree, ubtotal;
 	VolSpace gbfree, gbtotal;
 
-	uret = getfreespace(vol, &ubfree, &ubtotal,
-	    uuid, QUOTA_IDTYPE_USER);
-	if (uret == 1) {
-		LOG(log_info, logtype_afpd, "quota_get(%s, user): %d %d",
-		    vol->v_path, (int)ubfree, (int)ubtotal);
-	}
-
-	if (ngroups >= 1) {
-		gret = getfreespace(vol, &gbfree, &gbtotal,
-		    groups[0], QUOTA_IDTYPE_GROUP);
-		if (gret == 1) {
-			LOG(log_info, logtype_afpd, "quota_get(%s, group): %d %d",
-			    vol->v_path, (int)gbfree, (int)gbtotal);
-		}
+	uretq = getfreespace(obj, vol, &ubfree, &ubtotal,
+			     uuid, QUOTADICT_CLASS_USER);
+	LOG(log_info, logtype_afpd, "getfsquota(%s): %d %d",
+	    vol->v_path, (int)ubfree, (int)ubtotal);
+	if (obj->ngroups >= 1) {
+		gretq = getfreespace(vol, &ubfree, &ubtotal,
+		    obj->groups[0], QUOTADICT_CLASS_GROUP);
 	} else
-		gret = 0;
-
-	if (uret < 1 && gret < 1) { /* no quota for this fs */
+		gretq = -1;
+	if (uretq < 1 && gretq < 1) { /* no quota for this fs */
 		return AFPERR_PARAM;
 	}
-	if (uret < 1) {
-		/* no user quotas, but group quotas; use them */
+	if (uretq < 1) {
+		/* use group quotas */
 		*bfree = gbfree;
 		*btotal = gbtotal;
-	} else if (gret < 1) {
-		/* no group quotas, but user quotas; use them */
+	} else if (gretq < 1) {
+		/* use user quotas */
 		*bfree = ubfree;
 		*btotal = ubtotal;
 	} else {
-		/* both; return smallest remaining space of user and group */
+		/* return smallest remaining space of user and group */
 		if (ubfree < gbfree) {
 			*bfree = ubfree;
 			*btotal = ubtotal;
@@ -343,13 +275,13 @@ static int get_linux_xfs_quota(int what, char *path, uid_t euser_id, struct dqbl
 	if ((ret = quotactl(QCMD(Q_XGETQUOTA,(what ? GRPQUOTA : USRQUOTA)), path, euser_id, (caddr_t)&D)))
                return ret;
 
-	dqb->bsize = (u_int64_t)512;
-        dqb->dqb_bsoftlimit  = (u_int64_t)D.d_blk_softlimit;
-        dqb->dqb_bhardlimit  = (u_int64_t)D.d_blk_hardlimit;
-        dqb->dqb_ihardlimit  = (u_int64_t)D.d_ino_hardlimit;
-        dqb->dqb_isoftlimit  = (u_int64_t)D.d_ino_softlimit;
-        dqb->dqb_curinodes   = (u_int64_t)D.d_icount;
-        dqb->dqb_curblocks   = (u_int64_t)D.d_bcount; 
+	dqb->bsize = (uint64_t)512;
+        dqb->dqb_bsoftlimit  = (uint64_t)D.d_blk_softlimit;
+        dqb->dqb_bhardlimit  = (uint64_t)D.d_blk_hardlimit;
+        dqb->dqb_ihardlimit  = (uint64_t)D.d_ino_hardlimit;
+        dqb->dqb_isoftlimit  = (uint64_t)D.d_ino_softlimit;
+        dqb->dqb_curinodes   = (uint64_t)D.d_icount;
+        dqb->dqb_curblocks   = (uint64_t)D.d_bcount; 
 #endif
        return ret;
 }
@@ -438,7 +370,7 @@ mountp( char *file, int *nfs)
     dev_t			devno;
     static struct mnttab	mnt;
 
-    if (stat(file, &sb) < 0) {
+    if ( lstat( file, &sb ) < 0 ) {
         return( NULL );
     }
     devno = sb.st_dev;
@@ -449,14 +381,16 @@ mountp( char *file, int *nfs)
 
     while ( getmntent( mtab, &mnt ) == 0 ) {
         /* local fs */
-        if ( (stat( mnt.mnt_special, &sb ) == 0) && (devno == sb.st_rdev)) {
+        if ( (lstat( mnt.mnt_special, &sb ) == 0) && (devno == sb.st_rdev)) {
             fclose( mtab );
             return mnt.mnt_mountp;
         }
 
-        /* check for nfs. */
-        if ((stat(mnt.mnt_mountp, &sb) == 0) && (devno == sb.st_dev)) {
-            *nfs = strcmp(mnt.mnt_fstype, MNTTYPE_NFS) == 0 ? 1 : 0;
+        /* check for nfs. we probably should use
+         * strcmp(mnt.mnt_fstype, MNTTYPE_NFS), but that's not as fast. */
+        if ((lstat(mnt.mnt_mountp, &sb) == 0) && (devno == sb.st_dev) &&
+                strchr(mnt.mnt_special, ':')) {
+            *nfs = 1;
             fclose( mtab );
             return mnt.mnt_special;
         }
@@ -524,7 +458,7 @@ special(char *file, int *nfs)
     struct mntent	*mnt;
     int 		found=0;
 
-    if (stat(file, &sb) < 0 ) {
+    if ( lstat( file, &sb ) < 0 ) {
         return( NULL );
     }
     devno = sb.st_dev;
@@ -535,14 +469,14 @@ special(char *file, int *nfs)
 
     while (( mnt = getmntent( mtab )) != NULL ) {
         /* check for local fs */
-        if ( (stat( mnt->mnt_fsname, &sb ) == 0) && devno == sb.st_rdev) {
+        if ( (lstat( mnt->mnt_fsname, &sb ) == 0) && devno == sb.st_rdev) {
 	    found = 1;
 	    break;
         }
 
         /* check for an nfs mount entry. the alternative is to use
         * strcmp(mnt->mnt_type, MNTTYPE_NFS) instead of the strchr. */
-        if ((stat(mnt->mnt_dir, &sb) == 0) && (devno == sb.st_dev) &&
+        if ((lstat(mnt->mnt_dir, &sb) == 0) && (devno == sb.st_dev) &&
                 strchr(mnt->mnt_fsname, ':')) {
             *nfs = 1;
 	    found = 1;
@@ -567,7 +501,7 @@ special(char *file, int *nfs)
 #endif /* __svr4__ */
 
 
-static int getfsquota(struct vol *vol, const int uid, struct dqblk *dq)
+static int getfsquota(const AFPObj *obj, struct vol *vol, const int uid, struct dqblk *dq)
 
 {
 	struct dqblk dqg;
@@ -607,29 +541,19 @@ static int getfsquota(struct vol *vol, const int uid, struct dqblk *dq)
 #endif /* TRU64 */
 
 #ifdef BSD4_4
-    if ( seteuid( getuid() ) == 0 ) {
+    become_root();
         if ( quotactl( vol->v_path, QCMD(Q_GETQUOTA,USRQUOTA),
                        uid, (char *)dq ) != 0 ) {
             /* try group quotas */
-            if (ngroups >= 1) {
+            if (obj->ngroups >= 1) {
                 if ( quotactl(vol->v_path, QCMD(Q_GETQUOTA, GRPQUOTA),
-                              groups[0], (char *) &dqg) != 0 ) {
-                    seteuid( uid );
+                              obj->groups[0], (char *) &dqg) != 0 ) {
+                    unbecome_root();
                     return( AFPERR_PARAM );
                 }
             }
         }
-        seteuid( uid );
-    }
-
-#elif defined(TRU64)
-    if ( seteuid( getuid() ) == 0 ) {
-        if ( quotactl( vol->v_path, QCMD(Q_GETQUOTA, USRQUOTA),
-                       uid, (char *)dq ) != 0 ) {
-            seteuid( uid );
-            return ( AFPERR_PARAM );
-        }
-        seteuid( uid );
+        unbecome_root();
     }
 
 #else /* BSD4_4 */
@@ -656,12 +580,12 @@ static int getfsquota(struct vol *vol, const int uid, struct dqblk *dq)
     if( 
         /* if overquota, free space is 0 otherwise hard-current */
         ( overquota( dq ) ? 0 : ( dq->dqb_bhardlimit ? dq->dqb_bhardlimit - 
-                                  dq->dqb_curblocks : ~((u_int64_t) 0) ) )
+                                  dq->dqb_curblocks : ~((uint64_t) 0) ) )
 
       >
         
         ( overquota( &dqg ) ? 0 : ( dqg.dqb_bhardlimit ? dqg.dqb_bhardlimit - 
-                                    dqg.dqb_curblocks : ~((u_int64_t) 0) ) )
+                                    dqg.dqb_curblocks : ~((uint64_t) 0) ) )
 
       ) /* if */
     {
@@ -681,7 +605,7 @@ static int getfsquota(struct vol *vol, const int uid, struct dqblk *dq)
 }
 
 
-static int getquota( struct vol *vol, struct dqblk *dq, const u_int32_t bsize)
+static int getquota(const AFPObj *obj, struct vol *vol, struct dqblk *dq, const uint32_t bsize)
 {
     char *p;
 
@@ -756,11 +680,11 @@ static int getquota( struct vol *vol, struct dqblk *dq, const u_int32_t bsize)
 	    return getnfsquota(vol, uuid, bsize, dq);
     } else
 	/* local filesystem */
-	return getfsquota(vol, uuid, dq);
+      return getfsquota(obj, vol, obj->uid, dq);
 	   
 #else /* TRU64 */
-    return vol->v_nfs ? getnfsquota(vol, uuid, bsize, dq) :
-           getfsquota(vol, uuid, dq);
+    return vol->v_nfs ? getnfsquota(vol, obj->uid, bsize, dq) :
+      getfsquota(obj, vol, obj->uid, dq);
 #endif /* TRU64 */
 }
 
@@ -819,14 +743,14 @@ static int overquota( struct dqblk *dqblk)
 #define tobytes(a, b)  dbtob((VolSpace) (a))
 #endif
 
-int uquota_getvolspace( struct vol *vol, VolSpace *bfree, VolSpace *btotal, const u_int32_t bsize)
+int uquota_getvolspace(const AFPObj *obj, struct vol *vol, VolSpace *bfree, VolSpace *btotal, const uint32_t bsize)
 {
-	u_int64_t this_bsize;
+	uint64_t this_bsize;
 	struct dqblk dqblk;
 
 	this_bsize = bsize;
 			
-	if (getquota( vol, &dqblk, bsize) != 0 ) {
+	if (getquota(obj, vol, &dqblk, bsize) != 0 ) {
 		return( AFPERR_PARAM );
 	}
 
