@@ -1,6 +1,4 @@
 /*
- * $Id: uam_auth.h,v 1.4.6.1 2004/06/15 00:35:06 bfernhomberg Exp $
- *
  * Copyright (c) 1999 Adrian Sun (asun@zoology.washington.edu)
  * All Rights Reserved.  See COPYRIGHT.
  *
@@ -10,11 +8,10 @@
 #ifndef AFPD_UAM_AUTH_H
 #define AFPD_UAM_AUTH_H 1
 
-#include <sys/cdefs.h>
 #include <pwd.h>
 
 #include <atalk/uam.h>
-#include "globals.h"
+#include <atalk/globals.h>
 
 struct uam_mod {
     void *uam_module;
@@ -28,16 +25,16 @@ struct uam_obj {
     int uam_count;
     union {
         struct {
-            int (*login) __P((void *, struct passwd **,
-                              char *, int, char *, int *));
-            int (*logincont) __P((void *, struct passwd **, char *,
-                                  int, char *, int *));
-            void (*logout) __P((void));
-            int (*login_ext) __P((void *, char *, struct passwd **,
-                              char *, int, char *, int *));
+            int (*login) (void *, struct passwd **,
+                              char *, int, char *, size_t *);
+            int (*logincont) (void *, struct passwd **, char *,
+                                  int, char *, size_t *);
+            void (*logout) (void);
+            int (*login_ext) (void *, char *, struct passwd **,
+                              char *, int, char *, size_t *);
         } uam_login;
-        int (*uam_changepw) __P((void *, char *, struct passwd *, char *,
-                                 int, char *, int *));
+        int (*uam_changepw) (void *, char *, struct passwd *, char *,
+                                 int, char *, size_t *);
     } u;
     struct uam_obj *uam_prev, *uam_next;
 };
@@ -54,17 +51,17 @@ struct uam_obj {
     (a)->uam_next->uam_prev = (a)->uam_prev; \
 } while (0)
 
-extern struct uam_mod *uam_load __P((const char *, const char *));
-extern void uam_unload __P((struct uam_mod *));
+extern struct uam_mod *uam_load (const char *, const char *);
+extern void uam_unload (struct uam_mod *);
 
 /* auth.c */
-int auth_load __P((const char *, const char *));
-int auth_register __P((const int, struct uam_obj *));
+int auth_load (const char *, const char *);
+int auth_register (const int, struct uam_obj *);
 #define auth_unregister(a) uam_detach(a)
-struct uam_obj *auth_uamfind __P((const int, const char *, const int));
-void auth_unload __P((void));
+struct uam_obj *auth_uamfind (const int, const char *, const int);
+void auth_unload (void);
 
 /* uam.c */
-int uam_random_string __P((AFPObj *,char *, int));
+int uam_random_string (AFPObj *,char *, int);
 
 #endif /* uam_auth.h */
