@@ -12,49 +12,6 @@
    GNU General Public License for more details.
 */
 
-/*
-  dbd specs and implementation progress
-  =====================================
-
-  St := Status
-
-  Force option
-  ------------
-  
-  St Spec
-  -- ----
-  OK If -f is requested, ensure -e is too.
-     Check if volumes is using AFPVOL_CACHE, then wipe db from disk. Rebuild from ad-files.
-
-  1st pass: Scan volume
-  --------------------
-
-  St Type Check
-  -- ---- -----
-  OK F/D  Make sure ad file exists
-  OK D    Make sure .AppleDouble dir exist, create if missing. Error creating
-          it is fatal as that shouldn't happen as root.
-  OK F/D  Delete orphaned ad-files, log dirs in ad-dir
-  OK F/D  Check name encoding by roundtripping, log on error
-  OK F/D  try: read CNID from ad file (if cnid caching is on)
-          try: fetch CNID from database
-          -> on mismatch: use CNID from file, update database (deleting both found CNIDs first)
-          -> if no CNID in ad file: write CNID from database to ad file
-          -> if no CNID in database: add CNID from ad file to database
-          -> on no CNID at all: create one and store in both places
-  OK F/D  Add found CNID, DID, filename, dev/inode, stamp to rebuild database
-  OK F/D  Check/update stamp (implicitly done while checking CNIDs)
-
-
-  2nd pass: Delete unused CNIDs
-  -----------------------------
-
-  St Spec
-  -- ----
-  OK Step through dbd (the one on disk) and rebuild-db from pass 1 and delete any CNID from
-     dbd not in rebuild db. This in only done in exclusive mode.
-*/
-
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif /* HAVE_CONFIG_H */
@@ -71,18 +28,19 @@
 
 #include <atalk/logger.h>
 #include <atalk/cnid_dbd_private.h>
-#include <atalk/volinfo.h>
+#include <atalk/globals.h>
+#include <atalk/netatalk_conf.h>
 #include <atalk/util.h>
 
 #include "cmd_dbd.h"
 #include "dbd.h"
 #include "dbif.h"
 #include "db_param.h"
+#include "pack.h"
 
 #define DBOPTIONS (DB_CREATE | DB_INIT_LOCK | DB_INIT_LOG | DB_INIT_MPOOL | DB_INIT_TXN)
 
 int nocniddb = 0;               /* Dont open CNID database, only scan filesystem */
-struct volinfo volinfo; /* needed by pack.c:idxname() */
 volatile sig_atomic_t alarmed;  /* flags for signals */
 int db_locked;                  /* have we got the fcntl lock on lockfile ? */
 
@@ -171,47 +129,29 @@ static void set_signal(void)
 static void usage (void)
 {
     printf("dbd (Netatalk %s)\n"
-           "Usage: dbd [-e|-t|-v|-x] -d [-i] | -s [-c|-n]| -r [-c|-f] | -u <path to netatalk volume>\n"
+           "Usage: dbd [-CeFtvx] -d [-i] | -s [-c|-n]| -r [-c|-f] | -u <path to netatalk volume>\n"
            "dbd can dump, scan, reindex and rebuild Netatalk dbd CNID databases.\n"
-           "dbd must be run with appropriate permissions i.e. as root.\n\n"
+           "dbd must be run with appropiate permissions i.e. as root.\n\n"
            "Main commands are:\n"
            "   -d Dump CNID database\n"
            "      Option: -i dump indexes too\n\n"
            "   -s Scan volume:\n"
-           "      1. Compare CNIDs in database with volume\n"
-           "      2. Check if .AppleDouble dirs exist\n"
-           "      3. Check if  AppleDouble file exist\n"
-           "      4. Report orphaned AppleDouble files\n"
-           "      5. Check for directories inside AppleDouble directories\n"
-           "      6. Check name encoding by roundtripping, log on error\n"
-           "      7. Check for orphaned CNIDs in database (requires -e)\n"
-           "      8. Open and close adouble files\n"
-           "      Options: -c Don't check .AppleDouble stuff, only check orphaned.\n"
+           "      Options: -c Don't check .AppleDouble stuff, only ckeck orphaned.\n"
            "               -n Don't open CNID database, skip CNID checks.\n\n"
            "   -r Rebuild volume:\n"
-           "      1. Sync CNIDs in database with volume\n"
-           "      2. Make sure .AppleDouble dir exist, create if missing\n"
-           "      3. Make sure AppleDouble file exists, create if missing\n"
-           "      4. Delete orphaned AppleDouble files\n"
-           "      5. Check for directories inside AppleDouble directories\n"
-           "      6. Check name encoding by roundtripping, log on error\n"
-           "      7. Check for orphaned CNIDs in database (requires -e)\n"
-           "      8. Open and close adouble files\n"
            "      Options: -c Don't create .AppleDouble stuff, only cleanup orphaned.\n"
            "               -f wipe database and rebuild from IDs stored in AppleDouble\n"
-           "                  files, only available for volumes without 'nocnidcache'\n"
-           "                  option. Implies -e.\n\n"
+           "                  metadata file or EA. Implies -e.\n\n"
            "   -u Upgrade:\n"
            "      Opens the database which triggers any necessary upgrades,\n"
            "      then closes and exits.\n\n"
            "General options:\n"
+           "   -C convert from adouble:v2 to adouble:ea (use with -r)\n"
+           "   -F location of the afp.conf config file\n"
            "   -e only work on inactive volumes and lock them (exclusive)\n"
            "   -x rebuild indexes (just for completeness, mostly useless!)\n"
            "   -t show statistics while running\n"
            "   -v verbose\n\n"
-           "WARNING:\n"
-           "For -r -f restore of the CNID database from the adouble files,\n"
-           "the CNID must of course be synched to them files first with a plain -r rebuild!\n"
            , VERSION
         );
 }
@@ -223,6 +163,8 @@ int main(int argc, char **argv)
     dbd_flags_t flags = 0;
     char *volpath;
     int cdir;
+    AFPObj obj = { 0 };
+    struct vol *vol;
 
     if (geteuid() != 0) {
         usage();
@@ -231,11 +173,14 @@ int main(int argc, char **argv)
     /* Inhereting perms in ad_mkdir etc requires this */
     ad_setfuid(0);
 
-    while ((c = getopt(argc, argv, ":cdefinrstuvx")) != -1) {
+    while ((c = getopt(argc, argv, ":cCdefFinrstuvx")) != -1) {
         switch(c) {
         case 'c':
             flags |= DBD_FLAGS_CLEANUP;
             break;
+        case 'C':
+            flags |= DBD_FLAGS_V2TOEA;
+            break;
         case 'd':
             dump = 1;
             break;
@@ -273,6 +218,9 @@ int main(int argc, char **argv)
             exclusive = 1;
             flags |= DBD_FLAGS_FORCE | DBD_FLAGS_EXCL;
             break;
+        case 'F':
+            obj.cmdlineconfigfile = strdup(optarg);
+            break;
         case ':':
         case '?':
             usage();
@@ -303,47 +251,75 @@ int main(int argc, char **argv)
     /* Setup signal handling */
     set_signal();
 
-    /* Setup logging. Should be portable among *NIXes */
-    if (!verbose)
-        setuplog("default log_info /dev/tty");
-    else
-        setuplog("default log_debug /dev/tty");
-
-    /* Load .volinfo file */
-    if (loadvolinfo(volpath, &volinfo) == -1) {
-        dbd_log( LOGSTD, "Not a Netatalk volume at '%s', no .volinfo file at '%s/.AppleDesktop/.volinfo' or unknown volume options", volpath, volpath);
+    /* Load config */
+    if (afp_config_parse(&obj, "dbd") != 0) {
+        dbd_log( LOGSTD, "Couldn't load afp.conf");
         exit(EXIT_FAILURE);
     }
-    if (vol_load_charsets(&volinfo) == -1) {
-        dbd_log( LOGSTD, "Error loading charsets!");
+
+    /* Setup logging. Should be portable among *NIXes */
+    if (!verbose)
+        setuplog("default:info", "/dev/tty");
+    else
+        setuplog("default:debug", "/dev/tty");
+
+    if (load_volumes(&obj, NULL) != 0) {
+        dbd_log( LOGSTD, "Couldn't load volumes");
+        exit(EXIT_FAILURE);
+    }
+
+    if ((vol = getvolbypath(&obj, volpath)) == NULL) {
+        dbd_log( LOGSTD, "Couldn't find volume for '%s'", volpath);
+        exit(EXIT_FAILURE);
+    }
+
+    if (load_charset(vol) != 0) {
+        dbd_log( LOGSTD, "Couldn't load charsets for '%s'", volpath);
+        exit(EXIT_FAILURE);
+    }
+
+    pack_setvol(vol);
+
+    if (vol->v_adouble == AD_VERSION_EA)
+        dbd_log( LOGDEBUG, "adouble:ea volume");
+    else if (vol->v_adouble == AD_VERSION2)
+        dbd_log( LOGDEBUG, "adouble:v2 volume");
+    else {
+        dbd_log( LOGSTD, "unknown adouble volume");
+        exit(EXIT_FAILURE);
+    }
+
+    /* -C v2 to ea conversion only on adouble:ea volumes */
+    if ((flags & DBD_FLAGS_V2TOEA) && (vol->v_adouble!= AD_VERSION_EA)) {
+        dbd_log( LOGSTD, "Can't run adouble:v2 to adouble:ea conversion because not an adouble:ea volume");
         exit(EXIT_FAILURE);
     }
 
     /* Sanity checks to ensure we can touch this volume */
-    if (volinfo.v_vfs_ea != AFPVOL_EA_AD && volinfo.v_vfs_ea != AFPVOL_EA_SYS) {
-        dbd_log( LOGSTD, "Unknown Extended Attributes option: %u", volinfo.v_vfs_ea);
+    if (vol->v_vfs_ea != AFPVOL_EA_AD && vol->v_vfs_ea != AFPVOL_EA_SYS) {
+        dbd_log( LOGSTD, "Unknown Extended Attributes option: %u", vol->v_vfs_ea);
         exit(EXIT_FAILURE);        
     }
 
     /* Enuser dbpath is there, create if necessary */
     struct stat st;
-    if (stat(volinfo.v_dbpath, &st) != 0) {
+    if (stat(vol->v_dbpath, &st) != 0) {
         if (errno != ENOENT) {
-            dbd_log( LOGSTD, "Can't stat dbpath \"%s\": %s", volinfo.v_dbpath, strerror(errno));
+            dbd_log( LOGSTD, "Can't stat dbpath \"%s\": %s", vol->v_dbpath, strerror(errno));
             exit(EXIT_FAILURE);        
         }
-        if ((mkdir(volinfo.v_dbpath, 0755)) != 0) {
-            dbd_log( LOGSTD, "Can't create dbpath \"%s\": %s", dbpath, strerror(errno));
+        if ((mkdir(vol->v_dbpath, 0755)) != 0) {
+            dbd_log( LOGSTD, "Can't create dbpath \"%s\": %s", vol->v_dbpath, strerror(errno));
             exit(EXIT_FAILURE);
         }        
     }
 
     /* Put "/.AppleDB" at end of volpath, get path from volinfo file */
-    if ( (strlen(volinfo.v_dbpath) + strlen("/.AppleDB")) > MAXPATHLEN ) {
+    if ( (strlen(vol->v_dbpath) + strlen("/.AppleDB")) > MAXPATHLEN ) {
         dbd_log( LOGSTD, "Volume pathname too long");
         exit(EXIT_FAILURE);        
     }
-    strncpy(dbpath, volinfo.v_dbpath, MAXPATHLEN - strlen("/.AppleDB"));
+    strncpy(dbpath, vol->v_dbpath, MAXPATHLEN - strlen("/.AppleDB"));
     strcat(dbpath, "/.AppleDB");
 
     /* Check or create dbpath */
@@ -366,6 +342,7 @@ int main(int argc, char **argv)
     if ((db_locked = get_lock(LOCK_EXCL, dbpath)) == -1)
         goto exit_noenv;
     if (db_locked != LOCK_EXCL) {
+        dbd_log(LOGDEBUG, "Database is in use, acquiring shared lock");
         /* Couldn't get exclusive lock, try shared lock if -e wasn't requested */
         if (exclusive) {
             dbd_log(LOGSTD, "Database is in use and exlusive was requested");
@@ -376,7 +353,7 @@ int main(int argc, char **argv)
     }
 
     /* Check if -f is requested and wipe db if yes */
-    if ((flags & DBD_FLAGS_FORCE) && rebuild && (volinfo.v_flags & AFPVOL_CACHE)) {
+    if ((flags & DBD_FLAGS_FORCE) && rebuild) {
         char cmd[8 + MAXPATHLEN];
         if ((db_locked = get_lock(LOCK_FREE, NULL)) != 0)
             goto exit_noenv;
@@ -436,7 +413,7 @@ int main(int argc, char **argv)
             dbd_log( LOGSTD, "Error dumping database");
         }
     } else if ((rebuild && ! nocniddb) || scan) {
-        if (cmd_dbd_scanvol(dbd, &volinfo, flags) < 0) {
+        if (cmd_dbd_scanvol(dbd, vol, flags) < 0) {
             dbd_log( LOGSTD, "Error repairing database.");
         }
     }
