@@ -14,8 +14,8 @@
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <sys/cdefs.h>
 #include <unistd.h>
+#include <arpa/inet.h>
 
 #include <db.h>
 
@@ -111,7 +111,7 @@ static int dbif_init_rootinfo(DBD *dbd, int version)
     if (dbif_put(dbd, DBIF_CNID, &key, &data, 0) < 0)
         return -1;
     if (dbif_txn_commit(dbd) != 1) {
-        LOG(log_error, logtype_cnid, "dbif_init_rootinfo: can't commit txn");
+        LOG(log_error, logtype_cnid, "dbif_init_rootinfo: cant commit txn");
         return -1;
     }
 
@@ -347,6 +347,12 @@ int get_lock(int cmd, const char *dbpath)
     char lockpath[PATH_MAX];
     struct stat st;
 
+    LOG(log_debug, logtype_cnid, "get_lock(%s, \"%s\")",
+        cmd == LOCK_EXCL ? "LOCK_EXCL" :
+        cmd == LOCK_SHRD ? "LOCK_SHRD" :
+        cmd == LOCK_FREE ? "LOCK_FREE" :
+        cmd == LOCK_UNLOCK ? "LOCK_UNLOCK" : "UNKNOWN" , dbpath ? dbpath : "");
+
     switch (cmd) {
     case LOCK_FREE:
         if (lockfd == -1)
@@ -421,10 +427,8 @@ DBD *dbif_init(const char *envhome, const char *filename)
 
     /* filename == NULL means in memory db */
     if (filename) {
-        if (! envhome) {
-            free(dbd);
+        if (! envhome)
             return NULL;
-        }
 
         dbd->db_envhome = strdup(envhome);
         if (NULL == dbd->db_envhome) {
@@ -673,7 +677,7 @@ int dbif_open(DBD *dbd, struct db_param *dbp, int reindex)
                                       dbd->db_table[i].type,
                                       dbd->db_table[i].openflags,
                                       0664) < 0) {
-            LOG(log_error, logtype_cnid, "Can't open database");
+            LOG(log_error, logtype_cnid, "Cant open database");
             return -1;
         }
 
@@ -810,7 +814,7 @@ int dbif_env_remove(const char *path)
     LOG(log_debug, logtype_cnid, "Trying to remove BerkeleyDB environment");
 
     if (get_lock(LOCK_EXCL, path) != LOCK_EXCL) {
-        LOG(log_debug, logtype_cnid, "CNID db \"%s\" in use, can't remove BerkeleyDB environment", path);
+        LOG(log_debug, logtype_cnid, "CNID db \"%s\" in use, not removing BerkeleyDB environment", path);
         return 0;
     }
     
