@@ -8,36 +8,31 @@
 #include "config.h"
 #endif /* HAVE_CONFIG_H */
 
-#ifdef HAVE_UNISTD_H
 #include <unistd.h>
-#endif /* HAVE_UNISTD_H */
-#ifdef HAVE_FCNTL_H
 #include <fcntl.h>
-#endif /* HAVE_FCNTL_H */
 #include <stdio.h>
 #include <stdlib.h>
 #include <errno.h>
 #include <signal.h>
 #include <string.h>
-#ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
-#endif /* HAVE_SYS_TYPES_H */
 #include <sys/param.h>
-#ifdef HAVE_SYS_STAT_H
 #include <sys/stat.h>
-#endif /* HAVE_SYS_STAT_H */
 #include <time.h>
 #include <sys/file.h>
+#include <arpa/inet.h>
 
-#include <netatalk/endian.h>
 #include <atalk/cnid_dbd_private.h>
 #include <atalk/logger.h>
-#include <atalk/volinfo.h>
+#include <atalk/errchk.h>
+#include <atalk/bstrlib.h>
+#include <atalk/netatalk_conf.h>
 
 #include "db_param.h"
 #include "dbif.h"
 #include "dbd.h"
 #include "comm.h"
+#include "pack.h"
 
 /* 
    Note: DB_INIT_LOCK is here so we can run the db_* utilities while netatalk is running.
@@ -45,9 +40,6 @@
  */
 #define DBOPTIONS (DB_CREATE | DB_INIT_LOG | DB_INIT_MPOOL | DB_INIT_LOCK | DB_INIT_TXN)
 
-/* Global, needed by pack.c:idxname() */
-struct volinfo volinfo;
-
 static DBD *dbd;
 static int exit_sig = 0;
 static int db_locked;
@@ -247,7 +239,7 @@ static void switch_to_user(char *dir)
         exit(1);
     }
     if (!getuid()) {
-        LOG(log_info, logtype_cnid, "Setting uid/gid to %i/%i", st.st_uid, st.st_gid);
+        LOG(log_debug, logtype_cnid, "Setting uid/gid to %i/%i", st.st_uid, st.st_gid);
         if (setgid(st.st_gid) < 0 || setuid(st.st_uid) < 0) {
             LOG(log_error, logtype_cnid, "uid/gid: %s", strerror(errno));
             exit(1);
@@ -281,56 +273,79 @@ static void set_signal(void)
 /* ------------------------ */
 int main(int argc, char *argv[])
 {
+    EC_INIT;
     struct db_param *dbp;
-    int err = 0;
-    int ctrlfd, clntfd;
+    int delete_bdb = 0;
+    int ctrlfd = -1, clntfd = -1;
     char *logconfig;
+    AFPObj obj = { 0 };
+    struct vol *vol;
+    char *volpath = NULL;
+    bstring dbpath;
 
-    set_processname("cnid_dbd");
-
-    /* FIXME: implement -d from cnid_metad */
-    if (argc  != 5) {
-        LOG(log_error, logtype_cnid, "main: not enough arguments");
-        exit(1);
+    while (( ret = getopt( argc, argv, "dF:l:p:t:vV")) != -1 ) {
+        switch (ret) {
+        case 'd':
+            delete_bdb = 1;
+            break;
+        case 'F':
+            obj.cmdlineconfigfile = strdup(optarg);
+            break;
+        case 'p':
+            volpath = strdup(optarg);
+            break;
+        case 'l':
+            clntfd = atoi(optarg);
+            break;
+        case 't':
+            ctrlfd = atoi(optarg);
+            break;
+        case 'v':
+        case 'V':
+            printf("cnid_dbd (Netatalk %s)\n", VERSION);
+            return -1;
+        }
     }
 
-    ctrlfd = atoi(argv[2]);
-    clntfd = atoi(argv[3]);
-    logconfig = strdup(argv[4]);
-    setuplog(logconfig);
-
-    /* Load .volinfo file */
-    if (loadvolinfo(argv[1], &volinfo) == -1) {
-        LOG(log_error, logtype_cnid, "Cant load volinfo for \"%s\"", argv[1]);
+    if (ctrlfd == -1 || clntfd == -1 || !volpath) {
+        LOG(log_error, logtype_cnid, "main: bad IPC fds");
         exit(EXIT_FAILURE);
     }
-    /* Put "/.AppleDB" at end of volpath, get path from volinfo file */
-    char dbpath[MAXPATHLEN+1];
-    if ((strlen(volinfo.v_dbpath) + strlen("/.AppleDB")) > MAXPATHLEN ) {
-        LOG(log_error, logtype_cnid, "CNID db pathname too long: \"%s\"", volinfo.v_dbpath);
-        exit(EXIT_FAILURE);
-    }
-    strncpy(dbpath, volinfo.v_dbpath, MAXPATHLEN - strlen("/.AppleDB"));
-    strcat(dbpath, "/.AppleDB");
 
-    if (vol_load_charsets(&volinfo) == -1) {
-        LOG(log_error, logtype_cnid, "Error loading charsets!");
-        exit(EXIT_FAILURE);
-    }
-    LOG(log_debug, logtype_cnid, "db dir: \"%s\"", dbpath);
+    EC_ZERO( afp_config_parse(&obj, "cnid_dbd") );
 
-    switch_to_user(dbpath);
+    EC_ZERO( load_volumes(&obj, NULL) );
+    EC_NULL( vol = getvolbypath(&obj, volpath) );
+    EC_ZERO( load_charset(vol) );
+    pack_setvol(vol);
+
+    EC_NULL( dbpath = bfromcstr(vol->v_dbpath) );
+    EC_ZERO( bcatcstr(dbpath, "/.AppleDB") );
+
+    LOG(log_debug, logtype_cnid, "db dir: \"%s\"", bdata(dbpath));
+
+    switch_to_user(bdata(dbpath));
 
     /* Get db lock */
-    if ((db_locked = get_lock(LOCK_EXCL, dbpath)) == -1) {
+    if ((db_locked = get_lock(LOCK_EXCL, bdata(dbpath))) == -1) {
         LOG(log_error, logtype_cnid, "main: fatal db lock error");
-        exit(1);
+        EC_FAIL;
     }
     if (db_locked != LOCK_EXCL) {
         /* Couldn't get exclusive lock, try shared lock  */
         if ((db_locked = get_lock(LOCK_SHRD, NULL)) != LOCK_SHRD) {
             LOG(log_error, logtype_cnid, "main: fatal db lock error");
-            exit(1);
+            EC_FAIL;
+        }
+    }
+
+    if (delete_bdb && (db_locked == LOCK_EXCL)) {
+        LOG(log_warning, logtype_cnid, "main: too many CNID db opening attempts, wiping the slate clean");
+        chdir(bdata(dbpath));
+        system("rm -f cnid2.db lock log.* __db.*");
+        if ((db_locked = get_lock(LOCK_EXCL, bdata(dbpath))) != LOCK_EXCL) {
+            LOG(log_error, logtype_cnid, "main: fatal db lock error");
+            EC_FAIL;
         }
     }
 
@@ -339,59 +354,66 @@ int main(int argc, char *argv[])
     /* SIGINT and SIGTERM are always off, unless we are in pselect */
     block_sigs_onoff(1);
 
-    if ((dbp = db_param_read(dbpath)) == NULL)
-        exit(1);
+    if ((dbp = db_param_read(bdata(dbpath))) == NULL)
+        EC_FAIL;
     LOG(log_maxdebug, logtype_cnid, "Finished parsing db_param config file");
 
-    if (NULL == (dbd = dbif_init(dbpath, "cnid2.db")))
-        exit(2);
+    if (NULL == (dbd = dbif_init(bdata(dbpath), "cnid2.db")))
+        EC_FAIL;
 
     /* Only recover if we got the lock */
     if (dbif_env_open(dbd,
                       dbp,
                       (db_locked == LOCK_EXCL) ? DBOPTIONS | DB_RECOVER : DBOPTIONS) < 0)
-        exit(2); /* FIXME: same exit code as failure for dbif_open() */
+        EC_FAIL;
     LOG(log_debug, logtype_cnid, "Finished initializing BerkeleyDB environment");
 
     if (dbif_open(dbd, dbp, 0) < 0) {
-        dbif_close(dbd);
-        exit(2);
+        ret = -1;
+        goto close_db;
     }
+
     LOG(log_debug, logtype_cnid, "Finished opening BerkeleyDB databases");
 
     /* Downgrade db lock  */
     if (db_locked == LOCK_EXCL) {
         if (get_lock(LOCK_UNLOCK, NULL) != 0) {
-            dbif_close(dbd);
-            exit(2);
+            ret = -1;
+            goto close_db;
         }
+
         if (get_lock(LOCK_SHRD, NULL) != LOCK_SHRD) {
-            dbif_close(dbd);
-            exit(2);
+            ret = -1;
+            goto close_db;
         }
     }
 
 
     if (comm_init(dbp, ctrlfd, clntfd) < 0) {
-        dbif_close(dbd);
-        exit(3);
+        ret = -1;
+        goto close_db;
     }
 
-    if (loop(dbp) < 0)
-        err++;
+    if (loop(dbp) < 0) {
+        ret = -1;
+        goto close_db;
+    }
 
+close_db:
     if (dbif_close(dbd) < 0)
-        err++;
+        ret = -1;
 
-    if (dbif_env_remove(dbpath) < 0)
-        err++;
+    if (dbif_env_remove(bdata(dbpath)) < 0)
+        ret = -1;
 
-    if (err)
-        exit(4);
-    else if (exit_sig)
+EC_CLEANUP:
+    if (ret != 0)
+        exit(1);
+
+    if (exit_sig)
         LOG(log_info, logtype_cnid, "main: Exiting on signal %i", exit_sig);
     else
         LOG(log_info, logtype_cnid, "main: Idle timeout, exiting");
 
-    return 0;
+    EC_EXIT;
 }
