@@ -26,7 +26,7 @@
 #ifdef HAVE_PAM_PAM_APPL_H
 #include <pam/pam_appl.h>
 #endif
-
+#include <arpa/inet.h>
 
 #if defined(GNUTLS_DHX)
 #include <gnutls/openssl.h>
@@ -58,14 +58,14 @@
 /* the secret key */
 static CAST_KEY castkey;
 static struct passwd *dhxpwd;
-static u_int8_t randbuf[KEYSIZE];
+static uint8_t randbuf[KEYSIZE];
 
 /* diffie-hellman bits */
 static unsigned char msg2_iv[] = "CJalbert";
 static unsigned char msg3_iv[] = "LWallace";
-static const u_int8_t p[] = {0xBA, 0x28, 0x73, 0xDF, 0xB0, 0x60, 0x57, 0xD4,
+static const uint8_t p[] = {0xBA, 0x28, 0x73, 0xDF, 0xB0, 0x60, 0x57, 0xD4,
 			     0x3F, 0x20, 0x24, 0x74, 0x4C, 0xEE, 0xE7, 0x5B};
-static const u_int8_t g = 0x07;
+static const uint8_t g = 0x07;
 
 
 /* Static variables used to communicate between the conversation function
@@ -187,10 +187,9 @@ static struct pam_conv PAM_conversation = {
 static int dhx_setup(void *obj, char *ibuf, size_t ibuflen _U_, 
 		     char *rbuf, size_t *rbuflen)
 {
-    u_int16_t sessid;
+    uint16_t sessid;
     size_t i;
     BIGNUM *bn, *gbn, *pbn;
-    const BIGNUM *pub_key;
     DH *dh;
 
     /* get the client's public key */
@@ -234,16 +233,9 @@ static int dhx_setup(void *obj, char *ibuf, size_t ibuflen _U_,
       return AFPERR_PARAM;
     }
 
-    if (!DH_set0_pqg(dh, pbn, NULL, gbn)) {
-      BN_free(pbn);
-      BN_free(gbn);
-    /* Log Entry */
-      LOG(log_info, logtype_uams, "uams_dhx_pam.c :PAM DH_set0_pqg() mysteriously failed  -- %s", strerror(errno));
-    /* Log Entry */
-      goto pam_fail;
-    }
-
     /* generate key and make sure that we have enough space */
+    dh->p = pbn;
+    dh->g = gbn;
     if (DH_generate_key(dh) == 0) {
 	unsigned long dherror;
 	char errbuf[256];
@@ -257,17 +249,16 @@ static int dhx_setup(void *obj, char *ibuf, size_t ibuflen _U_,
 	ERR_free_strings();
 	goto pam_fail;
     }
-    DH_get0_key(dh, &pub_key, NULL);
-    if (BN_num_bytes(pub_key) > KEYSIZE) {
+    if (BN_num_bytes(dh->pub_key) > KEYSIZE) {
 	LOG(log_info, logtype_uams, "uams_dhx_pam.c :PAM: Err Generating Key -- Not enough Space? -- %s", strerror(errno));
 	goto pam_fail;
     }
 
     /* figure out the key. store the key in rbuf for now. */
-    i = DH_compute_key((unsigned char *)rbuf, bn, dh);
+    i = DH_compute_key(rbuf, bn, dh);
     
     /* set the key */
-    CAST_set_key(&castkey, i, (unsigned char *)rbuf);
+    CAST_set_key(&castkey, i, rbuf);
     
     /* session id. it's just a hashed version of the object pointer. */
     sessid = dhxhash(obj);
@@ -276,7 +267,7 @@ static int dhx_setup(void *obj, char *ibuf, size_t ibuflen _U_,
     *rbuflen += sizeof(sessid);
     
     /* public key */
-    BN_bn2bin(pub_key, (unsigned char *)rbuf); 
+    BN_bn2bin(dh->pub_key, rbuf); 
     rbuf += KEYSIZE;
     *rbuflen += KEYSIZE;
 
@@ -384,8 +375,9 @@ static int pam_login_ext(void *obj, char *uname, struct passwd **uam_pwd,
 		     char *rbuf, size_t *rbuflen)
 {
     char *username;
-    int len, ulen;
-    u_int16_t  temp16;
+    int len;
+    size_t ulen;
+    uint16_t  temp16;
 
     *rbuflen = 0;
 
@@ -421,7 +413,7 @@ static int pam_logincont(void *obj, struct passwd **uam_pwd,
 {
     const char *hostname;
     BIGNUM *bn1, *bn2, *bn3;
-    u_int16_t sessid;
+    uint16_t sessid;
     int err, PAM_error;
 
     *rbuflen = 0;
@@ -581,7 +573,7 @@ static int pam_changepw(void *obj, char *username,
     char *hostname;
     pam_handle_t *lpamh;
     uid_t uid;
-    u_int16_t sessid;
+    uint16_t sessid;
     int PAM_error;
 
     if (ibuflen < sizeof(sessid)) {
