@@ -12,42 +12,21 @@
 
 #include <stdio.h>
 #include <stdlib.h>
-
-/* STDC check */
-#if STDC_HEADERS
 #include <string.h>
-#else /* STDC_HEADERS */
-#ifndef HAVE_STRCHR
-#define strchr index
-#define strrchr index
-#endif /* HAVE_STRCHR */
-char *strchr (), *strrchr ();
-#ifndef HAVE_MEMCPY
-#define memcpy(d,s,n) bcopy ((s), (d), (n))
-#define memmove(d,s,n) bcopy ((s), (d), (n))
-#endif /* ! HAVE_MEMCPY */
-#endif /* STDC_HEADERS */
-
-#ifdef HAVE_UNISTD_H
 #include <unistd.h>
-#endif /* HAVE_UNISTD_H */
-#ifdef HAVE_FCNTL_H
 #include <fcntl.h>
-#endif /* HAVE_FCNTL_H */
 #include <ctype.h>
 #include <pwd.h>
 #include <sys/stat.h>
 #include <sys/param.h>
+#include <arpa/inet.h>
 
 #include <atalk/logger.h>
-
-#include <netatalk/endian.h>
-
 #include <atalk/afp.h>
 #include <atalk/uam.h>
 
 
-#include <openssl/des.h>
+#include <des.h>
 
 #ifdef USE_CRACKLIB
 #include <crack.h>
@@ -55,10 +34,10 @@ char *strchr (), *strrchr ();
 
 #define PASSWDLEN 8
 
-static DES_cblock	seskey;
-static DES_key_schedule	seskeysched;
+static C_Block		seskey;
+static Key_schedule	seskeysched;
 static struct passwd	*randpwd;
-static u_int8_t         randbuf[8];
+static uint8_t         randbuf[8];
 
 /* hash to a 16-bit number. this will generate completely harmless 
  * warnings on 64-bit machines. */
@@ -144,9 +123,9 @@ static int afppasswd(const struct passwd *pwd,
 		     unsigned char *passwd, int len, 
 		     const int set)
 {
-  u_int8_t key[DES_KEY_SZ*2];
+  uint8_t key[DES_KEY_SZ*2];
   char buf[MAXPATHLEN + 1], *p;
-  DES_key_schedule	schedule;
+  Key_schedule	schedule;
   FILE *fp;
   unsigned int i, j;
   int keyfd = -1, err = 0;
@@ -203,17 +182,17 @@ afppasswd_found:
 	key[j] = (unhex(key[i]) << 4) | unhex(key[i + 1]);
       if (j <= DES_KEY_SZ)
 	memset(key + j, 0, sizeof(key) - j);
-      DES_key_sched((DES_cblock *) key, &schedule);
+      key_sched((C_Block *) key, schedule);
       memset(key, 0, sizeof(key));
 
       if (set) {
 	/* NOTE: this takes advantage of the fact that passwd doesn't
 	 *       get used after this call if it's being set. */
-	DES_ecb_encrypt((DES_cblock *) passwd, (DES_cblock *) passwd, &schedule,
+	ecb_encrypt((C_Block *) passwd, (C_Block *) passwd, schedule,
 		    DES_ENCRYPT);
       } else {
 	/* decrypt the password */
-	DES_ecb_encrypt((DES_cblock *) p, (DES_cblock *) p, &schedule, DES_DECRYPT);
+	ecb_encrypt((C_Block *) p, (C_Block *) p, schedule, DES_DECRYPT);
       }
       memset(&schedule, 0, sizeof(schedule));
   }
@@ -305,7 +284,7 @@ static int rand_login(void *obj, char *username, int ulen, struct passwd **uam_p
 {
 
   char *passwdfile;
-  u_int16_t sessid;
+  uint16_t sessid;
   size_t len;
   int err;
  
@@ -350,7 +329,7 @@ static int randnum_logincont(void *obj, struct passwd **uam_pwd,
 			     char *ibuf, size_t ibuflen _U_, 
 			     char *rbuf _U_, size_t *rbuflen)
 {
-  u_int16_t sessid;
+  uint16_t sessid;
 
   *rbuflen = 0;
 
@@ -362,10 +341,10 @@ static int randnum_logincont(void *obj, struct passwd **uam_pwd,
 
   /* encrypt. this saves a little space by using the fact that
    * des can encrypt in-place without side-effects. */
-  DES_key_sched((DES_cblock *) seskey, &seskeysched);
+  key_sched((C_Block *) seskey, seskeysched);
   memset(seskey, 0, sizeof(seskey));
-  DES_ecb_encrypt((DES_cblock *) randbuf, (DES_cblock *) randbuf,
-	       &seskeysched, DES_ENCRYPT);
+  ecb_encrypt((C_Block *) randbuf, (C_Block *) randbuf,
+	       seskeysched, DES_ENCRYPT);
   memset(&seskeysched, 0, sizeof(seskeysched));
 
   /* test against what the client sent */
@@ -389,7 +368,7 @@ static int rand2num_logincont(void *obj, struct passwd **uam_pwd,
 			      char *ibuf, size_t ibuflen _U_, 
 			      char *rbuf, size_t *rbuflen)
 {
-  u_int16_t sessid;
+  uint16_t sessid;
   unsigned int i;
 
   *rbuflen = 0;
@@ -406,10 +385,10 @@ static int rand2num_logincont(void *obj, struct passwd **uam_pwd,
     seskey[i] <<= 1;
 
   /* encrypt randbuf */
-  DES_key_sched((DES_cblock *) seskey, &seskeysched);
+  key_sched((C_Block *) seskey, seskeysched);
   memset(seskey, 0, sizeof(seskey));
-  DES_ecb_encrypt((DES_cblock *) randbuf, (DES_cblock *) randbuf,
-	       &seskeysched, DES_ENCRYPT);
+  ecb_encrypt( (C_Block *) randbuf, (C_Block *) randbuf,
+	       seskeysched, DES_ENCRYPT);
 
   /* test against client's reply */
   if (memcmp(randbuf, ibuf, sizeof(randbuf))) { /* != */
@@ -421,8 +400,8 @@ static int rand2num_logincont(void *obj, struct passwd **uam_pwd,
   memset(randbuf, 0, sizeof(randbuf));
 
   /* encrypt client's challenge and send back */
-  DES_ecb_encrypt((DES_cblock *) ibuf, (DES_cblock *) rbuf,
-	       &seskeysched, DES_ENCRYPT);
+  ecb_encrypt( (C_Block *) ibuf, (C_Block *) rbuf,
+	       seskeysched, DES_ENCRYPT);
   memset(&seskeysched, 0, sizeof(seskeysched));
   *rbuflen = sizeof(randbuf);
   
@@ -457,15 +436,15 @@ static int randnum_changepw(void *obj, const char *username _U_,
       return err;
 
     /* use old passwd to decrypt new passwd */
-    DES_key_sched((DES_cblock *) seskey, &seskeysched);
+    key_sched((C_Block *) seskey, seskeysched);
     ibuf += PASSWDLEN; /* new passwd */
     ibuf[PASSWDLEN] = '\0';
-    DES_ecb_encrypt((DES_cblock *) ibuf, (DES_cblock *) ibuf, &seskeysched, DES_DECRYPT);
+    ecb_encrypt( (C_Block *) ibuf, (C_Block *) ibuf, seskeysched, DES_DECRYPT);
 
     /* now use new passwd to decrypt old passwd */
-    DES_key_sched((DES_cblock *) ibuf, &seskeysched);
+    key_sched((C_Block *) ibuf, seskeysched);
     ibuf -= PASSWDLEN; /* old passwd */
-    DES_ecb_encrypt((DES_cblock *) ibuf, (DES_cblock *) ibuf, &seskeysched, DES_DECRYPT);
+    ecb_encrypt((C_Block *) ibuf, (C_Block *) ibuf, seskeysched, DES_DECRYPT);
     if (memcmp(seskey, ibuf, sizeof(seskey))) 
 	err = AFPERR_NOTAUTH;
     else if (memcmp(seskey, ibuf + PASSWDLEN, sizeof(seskey)) == 0)
@@ -532,7 +511,7 @@ static int randnum_login_ext(void *obj, char *uname, struct passwd **uam_pwd,
 {
     char       *username;
     size_t     len, ulen;
-    u_int16_t  temp16;
+    uint16_t  temp16;
 
     *rbuflen = 0;
 
