@@ -1,7 +1,3 @@
-/*
- * $Id: cnid_tdb_get.c,v 1.1.2.1.2.1 2005/01/30 20:56:23 didg Exp $
- */
-
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -11,9 +7,9 @@
 #include "cnid_tdb.h"
 
 /* Return CNID for a given did/name. */
-cnid_t cnid_tdb_get(struct _cnid_db *cdb, const cnid_t did, char *name, const int len)
+cnid_t cnid_tdb_get(struct _cnid_db *cdb, cnid_t did, const char *name, size_t len)
 {
-    char start[TDB_DID_LEN + MAXPATHLEN + 1], *buf;
+    char start[CNID_DID_LEN + MAXPATHLEN + 1], *buf;
     struct _cnid_tdb_private *db;
     TDB_DATA key, data;
     cnid_t id;
@@ -31,7 +27,7 @@ cnid_t cnid_tdb_get(struct _cnid_db *cdb, const cnid_t did, char *name, const in
     memcpy(buf, name, len);
     *(buf + len) = '\0'; /* Make it a C-string. */
     key.dptr = start;
-    key.dsize = TDB_DID_LEN + len + 1;
+    key.dsize = CNID_DID_LEN + len + 1;
     data = tdb_fetch(db->tdb_didname, key);
     if (!data.dptr)
         return 0;
