@@ -30,8 +30,8 @@
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
+#include <arpa/inet.h>
 
-#include <netatalk/endian.h>
 #include <atalk/unicode.h>
 #include <atalk/logger.h>
 
@@ -48,6 +48,7 @@ struct charset_functions charset_mac_hebrew =
 	mac_hebrew_pull,
 	mac_hebrew_push,
 	CHARSET_CLIENT | CHARSET_MULTIBYTE,
+	NULL,
 	NULL, NULL
 };
 
@@ -80,7 +81,7 @@ char_ucs2_to_mac_hebrew ( unsigned char *r, ucs2_t wc)
     return 0;
 }
 
-static size_t mac_hebrew_push( void *cd, char **inbuf, size_t *inbytesleft,
+static size_t mac_hebrew_push( void *cd _U_, char **inbuf, size_t *inbytesleft,
                          char **outbuf, size_t *outbytesleft)
 {
     unsigned char c = 0;
@@ -152,7 +153,7 @@ char_mac_hebrew_to_ucs2 (ucs2_t *pwc, const unsigned char *s)
 	return 0;
 }
 
-static size_t mac_hebrew_pull ( void *cd, char **inbuf, size_t *inbytesleft,
+static size_t mac_hebrew_pull ( void *cd _U_, char **inbuf, size_t *inbytesleft,
                          char **outbuf, size_t *outbytesleft)
 {
     ucs2_t         temp;
