@@ -9,9 +9,7 @@
 #endif 
 
 #include <sys/types.h>
-#ifdef HAVE_UNISTD_H
 #include <unistd.h>
-#endif 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -71,8 +69,8 @@ static int ipc_kill_token(struct ipc_header *ipc, server_child *children)
 /* ----------------- */
 static int ipc_get_session(struct ipc_header *ipc, server_child *children)
 {
-    u_int32_t boottime;
-    u_int32_t idlen;
+    uint32_t boottime;
+    uint32_t idlen;
     char     *clientid, *p;
 
     
@@ -193,11 +191,11 @@ int reconnect_ipc(AFPObj *obj)
 
     while (retrycount++ < 10) {
         if ((obj->ipc_fd = ipc_client_uds(_PATH_AFP_IPC)) == -1) {
-            LOG(log_error, logtype_afpd, "reconnect_ipc: can't reconnect to master");
+            LOG(log_error, logtype_afpd, "reconnect_ipc: cant reconnect to master");
             sleep(1);
             continue;
         }
-        LOG(log_debug, logtype_afpd, "reconnect_ipc: successful IPC reconnect");
+        LOG(log_debug, logtype_afpd, "reconnect_ipc: succesfull IPC reconnect");
         return 0;
     }
     return -1;
@@ -349,4 +347,3 @@ int ipc_child_write(int fd, uint16_t command, int len, void *msg)
 
    return 0;
 }
-
